import sys
import os
import json
import logging
import ntpath
import glob
from PyQt5.QtWidgets import QTableWidgetItem, QFileDialog, QMessageBox
from PyQt5 import QtSql, QtGui, QtCore
from PyQt5.QtCore import QByteArray


"""
Application settings are stored in 3 json files. 
- configFields.json stores FITS keyword that users can
customize. Moreover an optional default value allows
users to fill FITS header that are missing.
- configFilters: filter keyword could be stored in FITS
header with different names. In order to match them all,
a list of comma separated value is allowed
- config.json: additional AstroDom configuration
"""


class SettingsTab:
    logger = logging.getLogger(__name__)

    def selectionchange(self, currentProfile):
        self.setProfile(currentProfile)

    def setProfile(self, currentProfile):
        # open config-profile
        profileFile = "profile-" + currentProfile + ".json"
        with open(os.path.join("astrodom/config", profileFile), "r") as configFields:
            self.cProfile = json.load(configFields)
            self.logger.debug("Read Config Fits File " + str(configFields))

        # Profile name
        self.mainW.ui.lineEditProfileName.setText(currentProfile)

        # FITS Headers keywords
        self.mainW.ui.lineEditFitsFile.setText(
            self.cProfile["file"]["fitsHeader"])
        self.mainW.ui.lineEditFitsTarget.setText(
            self.cProfile["target"]["fitsHeader"])
        self.mainW.ui.lineEditFitsFrame.setText(
            self.cProfile["frame"]["fitsHeader"])
        self.mainW.ui.lineEditFitsFilter.setText(
            self.cProfile["filter"]["fitsHeader"])
        self.mainW.ui.lineEditFitsExposure.setText(
            self.cProfile["exposure"]["fitsHeader"]
        )
        self.mainW.ui.lineEditFitsTemp.setText(
            self.cProfile["temp"]["fitsHeader"])
        self.mainW.ui.lineEditFitsXbinning.setText(
            self.cProfile["xbinning"]["fitsHeader"]
        )
        self.mainW.ui.lineEditFitsYbinning.setText(
            self.cProfile["ybinning"]["fitsHeader"]
        )
        self.mainW.ui.lineEditFitsSitelat.setText(
            self.cProfile["sitelat"]["fitsHeader"]
        )
        self.mainW.ui.lineEditFitsSitelong.setText(
            self.cProfile["sitelong"]["fitsHeader"]
        )
        self.mainW.ui.lineEditFitsRa.setText(self.cProfile["ra"]["fitsHeader"])
        self.mainW.ui.lineEditFitsDec.setText(
            self.cProfile["dec"]["fitsHeader"])
        self.mainW.ui.lineEditFitsDate.setText(
            self.cProfile["date"]["fitsHeader"])
        self.mainW.ui.lineEditFitsGain.setText(
            self.cProfile["gain"]["fitsHeader"])
        self.mainW.ui.lineEditFitsOffset.setText(
            self.cProfile["offset"]["fitsHeader"])

        # FITS header defaults
        self.mainW.ui.lineEditFitsFileDefault.setText(
            self.cProfile["file"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsTargetDefault.setText(
            self.cProfile["target"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsFrameDefault.setText(
            self.cProfile["frame"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsFilterDefault.setText(
            self.cProfile["filter"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsExposureDefault.setText(
            self.cProfile["exposure"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsTempDefault.setText(
            self.cProfile["temp"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsXbinningDefault.setText(
            self.cProfile["xbinning"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsYbinningDefault.setText(
            self.cProfile["ybinning"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsSitelatDefault.setText(
            self.cProfile["sitelat"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsSitelongDefault.setText(
            self.cProfile["sitelong"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsRaDefault.setText(
            self.cProfile["ra"]["fitsDefault"])
        self.mainW.ui.lineEditFitsDecDefault.setText(
            self.cProfile["dec"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsDateDefault.setText(
            self.cProfile["date"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsGainDefault.setText(
            self.cProfile["gain"]["fitsDefault"]
        )
        self.mainW.ui.lineEditFitsOffsetDefault.setText(
            self.cProfile["offset"]["fitsDefault"]
        )
        # Hide
        self.mainW.ui.checkBoxFitsFile.setChecked(
            bool(self.app.conf["file"]["hide"]))
        self.mainW.ui.checkBoxFitsTarget.setChecked(
            bool(self.app.conf["target"]["hide"])
        )
        self.mainW.ui.checkBoxFitsFrame.setChecked(
            bool(self.app.conf["frame"]["hide"]))
        self.mainW.ui.checkBoxFitsFilter.setChecked(
            bool(self.app.conf["filter"]["hide"])
        )
        self.mainW.ui.checkBoxFitsExposure.setChecked(
            bool(self.app.conf["exposure"]["hide"])
        )
        self.mainW.ui.checkBoxFitsTemp.setChecked(
            bool(self.app.conf["temp"]["hide"]))
        self.mainW.ui.checkBoxFitsXbinning.setChecked(
            bool(self.app.conf["xbinning"]["hide"])
        )
        self.mainW.ui.checkBoxFitsYbinning.setChecked(
            bool(self.app.conf["ybinning"]["hide"])
        )
        self.mainW.ui.checkBoxFitsSitelat.setChecked(
            bool(self.app.conf["sitelat"]["hide"])
        )
        self.mainW.ui.checkBoxFitsSitelong.setChecked(
            bool(self.app.conf["sitelong"]["hide"])
        )
        self.mainW.ui.checkBoxFitsRa.setChecked(
            bool(self.app.conf["ra"]["hide"]))
        self.mainW.ui.checkBoxFitsDec.setChecked(
            bool(self.app.conf["dec"]["hide"]))
        self.mainW.ui.checkBoxFitsDate.setChecked(
            bool(self.app.conf["date"]["hide"]))
        self.mainW.ui.checkBoxFitsGain.setChecked(
            bool(self.app.conf["gain"]["hide"]))
        self.mainW.ui.checkBoxFitsOffset.setChecked(
            bool(self.app.conf["offset"]["hide"])
        )

        self.mainW.ui.comboBoxCoordFormat.setCurrentText(
            self.cProfile["coordFormat"]["description"]
        )

    def __init__(self, mainW, app):
        super().__init__()
        self.mainW = mainW
        self.app = app

        # load profile
        profileFiles = glob.glob("astrodom/config/profile-*.json")
        for i, f in enumerate(profileFiles):
            f = ntpath.basename(f)
            f = os.path.splitext(f)[0]
            f = f.replace("profile-", "")
            self.mainW.ui.comboBoxProfile.addItem(f, i)

        self.mainW.ui.comboBoxProfile.setCurrentText(
            self.app.config["profile"])
        self.currentProfile = self.mainW.ui.comboBoxProfile.currentText()
        self.setProfile(self.currentProfile)

        # Filters Keywords
        self.mainW.ui.lineEditFilterL.setText(
            ",".join(self.app.confFilters["L"]))
        self.mainW.ui.lineEditFilterR.setText(
            ",".join(self.app.confFilters["R"]))
        self.mainW.ui.lineEditFilterG.setText(
            ",".join(self.app.confFilters["G"]))
        self.mainW.ui.lineEditFilterB.setText(
            ",".join(self.app.confFilters["B"]))
        self.mainW.ui.lineEditFilterHa.setText(
            ",".join(self.app.confFilters["Ha"]))
        self.mainW.ui.lineEditFilterSii.setText(
            ",".join(self.app.confFilters["Sii"]))
        self.mainW.ui.lineEditFilterOiii.setText(
            ",".join(self.app.confFilters["Oiii"]))
        self.mainW.ui.lineEditFilterLpr.setText(
            ",".join(self.app.confFilters["LPR"]))

        # Config main settings
        self.mainW.ui.lineEditDbname.setText(self.app.config["dbname"])
        self.mainW.ui.comboBoxDebug.setItemText(10, "DEBUG")
        self.mainW.ui.comboBoxDebug.setCurrentText(self.app.config["debug"])

        self.mainW.ui.comboBoxMonthsFilter.setCurrentText(
            self.app.config["monthsFilter"]
        )
        self.mainW.ui.spinBoxDefaultTimeStart.setValue(
            self.app.config["defaultTimeStart"]
        )
        self.mainW.ui.spinBoxDefaultTimeEnd.setValue(
            self.app.config["defaultTimeEnd"])
        self.mainW.ui.spinBoxPrecision.setValue(self.app.config["precision"])

    def selectDb(self):
        dbName, _ = QFileDialog.getOpenFileName(
            self.mainW,
            "Select a SQLite file to open…",
            os.path.join(self.app.directory, "config"),
            "SQLite Files (*.db) ;; All Files (*)",
        )
        if dbName:
            with open(dbName) as fh:
                dbName = os.path.splitext(ntpath.basename(dbName))[0]
                self.mainW.ui.lineEditDbname.setText(dbName)
                self.logger.info(f"Selected {dbName}")

    def saveProfile(self):

        self.app.conf["file"]["fitsHeader"] = self.mainW.ui.lineEditFitsFile.text()
        self.app.conf["target"]["fitsHeader"] = self.mainW.ui.lineEditFitsTarget.text()
        self.app.conf["frame"]["fitsHeader"] = self.mainW.ui.lineEditFitsFrame.text()
        self.app.conf["filter"]["fitsHeader"] = self.mainW.ui.lineEditFitsFilter.text()
        self.app.conf["exposure"][
            "fitsHeader"
        ] = self.mainW.ui.lineEditFitsExposure.text()
        self.app.conf["temp"]["fitsHeader"] = self.mainW.ui.lineEditFitsTemp.text()
        self.app.conf["xbinning"][
            "fitsHeader"
        ] = self.mainW.ui.lineEditFitsXbinning.text()
        self.app.conf["ybinning"][
            "fitsHeader"
        ] = self.mainW.ui.lineEditFitsYbinning.text()
        self.app.conf["sitelat"][
            "fitsHeader"
        ] = self.mainW.ui.lineEditFitsSitelat.text()
        self.app.conf["sitelong"][
            "fitsHeader"
        ] = self.mainW.ui.lineEditFitsSitelong.text()
        self.app.conf["ra"]["fitsHeader"] = self.mainW.ui.lineEditFitsRa.text()
        self.app.conf["dec"]["fitsHeader"] = self.mainW.ui.lineEditFitsDec.text()
        self.app.conf["date"]["fitsHeader"] = self.mainW.ui.lineEditFitsDate.text()
        self.app.conf["gain"]["fitsHeader"] = self.mainW.ui.lineEditFitsGain.text()
        self.app.conf["offset"]["fitsHeader"] = self.mainW.ui.lineEditFitsOffset.text()

        self.app.conf["file"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsFileDefault.text()
        self.app.conf["target"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsTargetDefault.text()
        self.app.conf["frame"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsFrameDefault.text()
        self.app.conf["filter"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsFilterDefault.text()
        self.app.conf["exposure"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsExposureDefault.text()
        self.app.conf["temp"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsTempDefault.text()
        self.app.conf["xbinning"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsXbinningDefault.text()
        self.app.conf["ybinning"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsYbinningDefault.text()
        self.app.conf["sitelat"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsSitelatDefault.text()
        self.app.conf["sitelong"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsSitelongDefault.text()
        self.app.conf["ra"]["fitsDefault"] = self.mainW.ui.lineEditFitsRaDefault.text()
        self.app.conf["dec"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsDecDefault.text()
        self.app.conf["date"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsDateDefault.text()
        self.app.conf["gain"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsGainDefault.text()
        self.app.conf["offset"][
            "fitsDefault"
        ] = self.mainW.ui.lineEditFitsOffsetDefault.text()

        self.app.conf["coordFormat"][
            "description"
        ] = self.mainW.ui.comboBoxCoordFormat.currentText()
        # CheckBox hide

        self.app.conf["file"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsFile.isChecked() else 0
        )
        self.app.conf["target"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsTarget.isChecked() else 0
        )
        self.app.conf["frame"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsFrame.isChecked() else 0
        )
        self.app.conf["filter"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsFilter.isChecked() else 0
        )
        self.app.conf["exposure"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsExposure.isChecked() else 0
        )
        self.app.conf["temp"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsTemp.isChecked() else 0
        )
        self.app.conf["xbinning"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsXbinning.isChecked() else 0
        )
        self.app.conf["ybinning"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsYbinning.isChecked() else 0
        )
        self.app.conf["sitelat"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsSitelat.isChecked() else 0
        )
        self.app.conf["sitelong"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsSitelong.isChecked() else 0
        )
        self.app.conf["ra"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsRa.isChecked() else 0
        )
        self.app.conf["dec"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsDec.isChecked() else 0
        )
        self.app.conf["date"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsDate.isChecked() else 0
        )
        self.app.conf["gain"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsGain.isChecked() else 0
        )
        self.app.conf["offset"]["hide"] = (
            1 if self.mainW.ui.checkBoxFitsOffset.isChecked() else 0
        )

        fileName = self.mainW.ui.lineEditProfileName.text()
        if fileName == "":
            fileName = self.app.conf["profile"]
        fileName = "profile-" + fileName + ".json"

        # Clear the tableview state else it prevents hide/show
        # columns set here to work properly.
        self.app.settings.setValue("readStateOnStart", "False")

        with open(os.path.join(self.app.directory, "config", fileName), "w") as outfile:
            json.dump(self.app.conf, outfile)
            self.logger.debug(self.app.conf)

        self.saveConfig()

    def saveFilter(self):

        self.app.confFilters["L"] = self.mainW.ui.lineEditFilterL.text().split(
            ",")
        self.app.confFilters["R"] = self.mainW.ui.lineEditFilterR.text().split(
            ",")
        self.app.confFilters["G"] = self.mainW.ui.lineEditFilterG.text().split(
            ",")
        self.app.confFilters["B"] = self.mainW.ui.lineEditFilterB.text().split(
            ",")
        self.app.confFilters["Ha"] = self.mainW.ui.lineEditFilterHa.text().split(
            ",")
        self.app.confFilters["Sii"] = self.mainW.ui.lineEditFilterSii.text().split(
            ",")
        self.app.confFilters["Oiii"] = self.mainW.ui.lineEditFilterOiii.text().split(
            ","
        )
        self.app.confFilters["LPR"] = self.mainW.ui.lineEditFilterLpr.text().split(
            ",")
        with open(
            os.path.join(self.app.directory, "config",
                         "configFilters.json"), "w"
        ) as outfile2:
            json.dump(self.app.confFilters, outfile2)
            self.logger.debug(self.app.confFilters)

    def saveConfig(self):

        if len(self.mainW.ui.lineEditDbname.text()) == 0:
            QMessageBox.about(
                None,
                "Message",
                "Enter a name to create a new database or choose an existing one.",
            )
            return

        self.app.config["dbname"] = self.mainW.ui.lineEditDbname.text()
        self.app.config["debug"] = self.mainW.ui.comboBoxDebug.currentText()
        self.app.config["profile"] = self.mainW.ui.lineEditProfileName.text()
        self.mainW.changeProfileSig.emit(
            self.mainW.ui.lineEditProfileName.text())
        self.mainW.changeDbSig.emit(self.mainW.ui.lineEditDbname.text())

        self.app.config[
            "monthsFilter"
        ] = self.mainW.ui.comboBoxMonthsFilter.currentText()

        self.app.config[
            "defaultTimeStart"
        ] = self.mainW.ui.spinBoxDefaultTimeStart.value()

        self.app.config["defaultTimeEnd"] = self.mainW.ui.spinBoxDefaultTimeEnd.value()
        self.app.config["precision"] = self.mainW.ui.spinBoxPrecision.value()
        
        with open(
            os.path.join(self.app.directory, "config", "config.json"), "w"
        ) as outfile3:
            json.dump(self.app.config, outfile3)
            self.logger.debug(self.app.config)

        # Connect to selected database
        self.app.db.close()
        self.app.db.setDatabaseName(
            os.path.join(
                self.app.directory, "config", self.app.config["dbname"] + ".db"
            )
        )

        if not self.app.db.open():
            self.logger.critical("Failed to set up db connection")
        self.app.setUpDbStructure()

        # Force image list to reload data
        self.mainW.imageSourceModel.select()
        while self.mainW.imageSourceModel.canFetchMore():
            self.mainW.imageSourceModel.fetchMore()
        self.mainW.filterRegExpChanged()



# -----------------------------------------------------------
# AstrDom:
#
# (C) 2020 Ferrante Enriques, ferrante.enriques@gmail.com
# Released under GNU Public License (GPL)
# -----------------------------------------------------------

import sys
import os
import ntpath
import logging
import csv

from astropy import units as u
from astropy.coordinates import Angle, Longitude, Latitude

from datetime import datetime
from PyQt5 import QtSql
from PyQt5 import QtCore, QtGui
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import QByteArray
from PyQt5.QtGui import QColor

from dateutil.relativedelta import relativedelta

"""
The table view containing the image list. It reads
the database and set it as a source model. Then
the model's view is handled through a proxy model.
So main objects in this class are:
- The source model:  imageSourceModel (SqlTableModel)
- The proxy model: imageListModel (instance of SortFilterProxyModel)
- The table view: tableViewImages
Keep in mind that some methods are defined in the parent
MainW class
"""


class ImageListTab:

    logger = logging.getLogger(__name__)

    def __init__(self, mainW, app):
        super().__init__()
        self.mainW = mainW
        self.app = app

        self.displayImageList()

    def displayImageList(self):

        # Table Source model
        self.mainW.imageSourceModel = QtSql.QSqlTableModel()
        self.mainW.imageSourceModel.setTable("images")
        self.mainW.imageSourceModel.setSort(3, QtCore.Qt.DescendingOrder)

        self.mainW.imageSourceModel.select()
        while self.mainW.imageSourceModel.canFetchMore():
            self.mainW.imageSourceModel.fetchMore()

        # Proxy model used for filtering and sorting.
        self.mainW.imageListModel.setDynamicSortFilter(True)
        self.mainW.imageListModel.setSourceModel(self.mainW.imageSourceModel)

        for i, label in enumerate(self.app.filterDictToList("description")):
            self.mainW.imageListModel.setHeaderData(
                i, QtCore.Qt.Horizontal, label)

        # restore the header state only if it is allowed
        if self.app.settings.value("readStateOnStart") == "True":
            try:
                rs = QByteArray.fromHex(
                    bytes(self.app.settings.value("imageListState"), 'ascii'))
                self.mainW.ui.tableViewImages.horizontalHeader().restoreState(rs)
            except Exception as e:
                self.logger.error(f"Could not restore table header state {e}")
        else:
            self.app.settings.setValue("readStateOnStart", "True")

        self.mainW.ui.tableViewImages.setWordWrap(False)
        #self.mainW.ui.tableViewImages.verticalHeader().hide()
        self.mainW.ui.tableViewImages.setSortingEnabled(True)
        self.mainW.ui.tableViewImages.setAlternatingRowColors(True)
        self.mainW.ui.tableViewImages.setModel(self.mainW.imageListModel)
        self.mainW.ui.tableViewImages.setTextElideMode(QtCore.Qt.ElideLeft)
        self.mainW.ui.tableViewImages.setEditTriggers(
            QtWidgets.QAbstractItemView.NoEditTriggers
        )

        fd = FileDelegate(self.mainW.ui.tableViewImages)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(1, fd)

        framed = FrameDelegate(self.mainW.ui.tableViewImages)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(4, framed)

        filterd = FilterDelegate(self.mainW.ui.tableViewImages)
        filterd.getFilters(self.app.confFilters)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(5, filterd)
        dd = DateDelegate(self.mainW.ui.tableViewImages)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(16, dd)
        rd = RoundDelegate(self.mainW.ui.tableViewImages)
        rd.getPrecision(self.app.config["precision"])
        # Site Long and Site Lat have dms representation
        latd = LatDelegate(self.mainW.ui.tableViewImages)
        longd = LongDelegate(self.mainW.ui.tableViewImages)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(10, latd)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(11, longd)

        # RA has hms representation
        rad = HmsDelegate(self.mainW.ui.tableViewImages)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(12, rad)
        # DEC, ALT, AZ  have dms reprentation
        dad = DmsDelegate(self.mainW.ui.tableViewImages)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(13, dad)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(14, dad)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(15, dad)

        self.mainW.ui.tableViewImages.setItemDelegateForColumn(25, rd)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(26, rd)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(27, rd)
        self.mainW.ui.tableViewImages.setItemDelegateForColumn(28, rd)

        hideColList = self.app.filterDictToList("hide", "hide")
        for col, val in enumerate(hideColList):
            if val > 0:
                self.mainW.ui.tableViewImages.hideColumn(col)

    def deleteRows(self):
        indexes = self.mainW.ui.tableViewImages.selectedIndexes()

        if indexes:
            ret = QMessageBox.question(None,
                                       "Delete rows",
                                       "Are you sure to delete selected rows?",
                                       QMessageBox.Yes | QMessageBox.No
                                       )

            if ret == QMessageBox.Yes:
                self.mainW.imageListModel.removeSelectedRows(indexes, 1, None)
                # Force image list reload data
                self.mainW.imageSourceModel.select()
                while self.mainW.imageSourceModel.canFetchMore():
                    self.mainW.imageSourceModel.fetchMore()
                self.mainW.filterRegExpChanged()
    def exportDataCsv(self):
 
        fileName, _ = QtWidgets.QFileDialog.getSaveFileName(None, "Save File",
                                                            (QtCore.QDir.homePath() +"/export" + ".csv"), "CSV Files (*.csv)")
        if fileName:
            f = open(fileName, 'w')
            with f:
                writer = csv.writer(f, delimiter=",")
                headers=[]
                for headerColumn in range(self.mainW.imageListModel.columnCount()):
                    headers.append( self.mainW.imageListModel.headerData(headerColumn, QtCore.Qt.Horizontal))
                writer.writerow(headers)
                
                for rowNumber in range(self.mainW.imageListModel.rowCount()):
                    fields = [self.mainW.imageListModel.data(self.mainW.imageListModel.index(rowNumber, columnNumber),
                                              QtCore.Qt.DisplayRole)
                              for columnNumber in range(self.mainW.imageListModel.columnCount())]
                    writer.writerow(fields)



class FileDelegate(QtWidgets.QStyledItemDelegate):

    def displayText(self, value, locale):
        self.value = ntpath.basename(value)
        return super(FileDelegate, self).displayText(self.value, locale)

# Here  all delegate classes for column formatting

class FilterDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        self.value = value
        return super(FilterDelegate, self).displayText(self.value, locale)

    def initStyleOption(self, option, index):
        super(FilterDelegate, self).initStyleOption(option, index)
        if not index.parent().isValid():
            if self.value in self.filters["L"]:
                option.backgroundBrush = QtGui.QBrush(
                    QColor(244, 244, 244, 55))
            elif self.value in self.filters["R"]:
                option.backgroundBrush = QtGui.QBrush(QColor(255, 0, 0, 55))
            elif self.value in self.filters["B"]:
                option.backgroundBrush = QtGui.QBrush(QColor(55, 55, 255, 55))
            elif self.value in self.filters["G"]:
                option.backgroundBrush = QtGui.QBrush(QColor(0, 140, 55, 55))
            elif self.value in self.filters["Ha"]:
                option.backgroundBrush = QtGui.QBrush(QColor(190, 255, 0, 55))
            elif self.value in self.filters["Oiii"]:
                option.backgroundBrush = QtGui.QBrush(
                    QColor(150, 200, 255, 55))
            elif self.value in self.filters["Sii"]:
                option.backgroundBrush = QtGui.QBrush(
                    QColor(255, 120, 190, 55))

    def getFilters(self, filters):
        self.filters = filters
    

class FrameDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        self.value = value
        return super(FrameDelegate, self).displayText(self.value, locale)

    def initStyleOption(self, option, index):
        super(FrameDelegate, self).initStyleOption(option, index)
        if not index.parent().isValid():
            if self.value == "Light":
                option.backgroundBrush = QtGui.QBrush(
                    QColor(120, 120, 244, 55))
            elif self.value == "Dark":
                option.backgroundBrush = QtGui.QBrush(QColor(80, 80, 80, 55))
            elif self.value == "Flat":
                option.backgroundBrush = QtGui.QBrush(QColor(80, 80,  160, 55))
            elif self.value == "DarkFlat":
                option.backgroundBrush = QtGui.QBrush(
                    QColor(80, 80, 80, 55))
            elif self.value == "Bias":
                option.backgroundBrush = QtGui.QBrush(
                    QColor(55, 55, 55, 55))


class RoundDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        value = round(float(value), self.precision)
        return super(RoundDelegate, self).displayText(value, locale)

    def getPrecision(self, precision):
        self.precision = precision


class DateDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        utcDate = datetime.strptime(value, "%Y-%m-%dT%H:%M:%S")
        value = utcDate.strftime("%Y-%m-%d  %H:%M:%S")
        return super(DateDelegate, self).displayText(value, locale)


class HmsDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        a = Angle(value, u.deg)
        value = str(a.to_string(unit=u.hour, precision=0, sep=("h", "m", "s")))
        return super(HmsDelegate, self).displayText(value, locale)

    def initStyleOption(self, option, index):
        super(HmsDelegate, self).initStyleOption(option, index)
        option.displayAlignment = QtCore.Qt.AlignRight


class DmsDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        a = Angle(value, u.deg)
        value = str(a.to_string(unit=u.degree,
                                precision=0, sep=("°", "'", "''")))
        return super(DmsDelegate, self).displayText(value, locale)

    def initStyleOption(self, option, index):
        super(DmsDelegate, self).initStyleOption(option, index)
        option.displayAlignment = QtCore.Qt.AlignRight


class LongDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        a = Longitude(value, u.deg)
        a.wrap_angle = 180 * u.deg
        value = str(a.to_string(unit=u.degree,
                                precision=0, sep=("°", "'", "''")))
        return super(LongDelegate, self).displayText(value, locale)

    def initStyleOption(self, option, index):
        super(LongDelegate, self).initStyleOption(option, index)
        option.displayAlignment = QtCore.Qt.AlignRight


class LatDelegate(QtWidgets.QStyledItemDelegate):
    def displayText(self, value, locale):
        a = Latitude(value, u.deg)
        a.wrap_angle = 90 * u.deg
        value = str(a.to_string(unit=u.degree,
                                precision=0, sep=("°", "'", "''")))
        return super(LatDelegate, self).displayText(value, locale)

    def initStyleOption(self, option, index):
        super(LatDelegate, self).initStyleOption(option, index)
        option.displayAlignment = QtCore.Qt.AlignRight



from PyQt5 import QtSql
from PyQt5.QtWidgets import QDialog, QDataWidgetMapper
import sys
import logging
import matplotlib.pyplot as plt

from astropy.io import fits
from astropy.visualization import *
from astropy import units as u
from astropy.coordinates import Angle, Longitude, Latitude

from matplotlib.colors import LogNorm
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as Canvas
from matplotlib.figure import Figure

from .gui.imageDetailWindowGui import *

"""
When an image is double clicked on the image list
view, a mapper is used to show all informations about
that image.
Mapping a model implies that here the model can be 
edited / saved.
Matplot lib is used to show a not so well stretched 
preview of the image.
"""


class ImageDetailWindow(QDialog):
    logger = logging.getLogger(__name__)

    def __init__(self, app, imageListModel):
        super().__init__()
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.app = app
        self.setWindowTitle("Image Detail")
        self.imageListModel = imageListModel

    def plot(self, modelIndex):
        self.mapper = QDataWidgetMapper(self)
        self.mapper.setModel(self.imageListModel)
        self.mapper.addMapping(self.ui.lineEditFile, 1)
        self.mapper.addMapping(self.ui.lineEditHash, 2)
        self.mapper.addMapping(self.ui.lineEditTarget, 3)
        self.mapper.addMapping(self.ui.lineEditFrame, 4)
        self.mapper.addMapping(self.ui.lineEditFilter, 5)
        self.mapper.addMapping(self.ui.lineEditExposure, 6)
        self.mapper.addMapping(self.ui.lineEditTemp, 7)
        self.mapper.addMapping(self.ui.lineEditXbinning, 8)
        self.mapper.addMapping(self.ui.lineEditYbinning, 9)
        self.mapper.addMapping(self.ui.lineEditSiteLat, 10)
        self.mapper.addMapping(self.ui.lineEditSiteLong, 11)
        self.mapper.addMapping(self.ui.lineEditRa, 12)
        self.mapper.addMapping(self.ui.lineEditDec, 13)
        self.mapper.addMapping(self.ui.lineEditAlt, 14)
        self.mapper.addMapping(self.ui.lineEditAz, 15)
        self.mapper.addMapping(self.ui.lineEditDate, 16)
        self.mapper.addMapping(self.ui.lineEditGain, 17)
        self.mapper.addMapping(self.ui.lineEditOffset, 18)
        self.mapper.addMapping(self.ui.lineEditSubframeScale, 19)
        self.mapper.addMapping(self.ui.lineEditCameraGain, 20)
        self.mapper.addMapping(self.ui.lineEditCameraResolution, 21)
        self.mapper.addMapping(self.ui.lineEditScaleUnit, 22)
        self.mapper.addMapping(self.ui.lineEditDataUnit, 23)
        self.mapper.addMapping(self.ui.lineEditCsvFile, 24)
        self.mapper.addMapping(self.ui.lineEditFwhm, 25)
        self.mapper.addMapping(self.ui.lineEditEccentricity, 26)
        self.mapper.addMapping(self.ui.lineEditSnrWeight, 27)
        self.mapper.addMapping(self.ui.lineEditNoise, 28)
        self.mapper.setCurrentIndex(modelIndex.row())
        self.ui.lineEditRaF.setText(self.convertCoord(self.ui.lineEditRa.text(), "hms"))
        self.ui.lineEditDecF.setText(self.convertCoord(self.ui.lineEditDec.text(), "dms"))
        self.ui.lineEditSiteLatF.setText(self.convertCoord(self.ui.lineEditSiteLat.text(), "lat"))
        self.ui.lineEditSiteLongF.setText(self.convertCoord(self.ui.lineEditSiteLong.text(), "long"))
        self.ui.lineEditAltF.setText(self.convertCoord(self.ui.lineEditAlt.text(), "dms"))
        self.ui.lineEditAzF.setText(self.convertCoord(self.ui.lineEditAz.text(), "dms"))
        
        cr = self.imageListModel.index(modelIndex.row(), 1)
        t = self.imageListModel.data(cr, QtCore.Qt.DisplayRole)
        self.mplwidget = MatplotlibWidget(self.ui.MplWidget)
        self.mplwidget.setFileName(t)
        self.mplwidget.plot()
        try:
            self.resize(self.app.settings.value("sizeDetailW"))
            self.move(self.app.settings.value("posDetailW"))
        except Exception as e:
            self.logger.error(f"{e}")
        
  
        self.show()
        
    def closeEvent(self, event):
        self.app.settings.setValue("sizeDetailW", self.size())
        self.app.settings.setValue("posDetailW", self.pos())
        try:
            self.close()
        except Exception as e:
            self.logger.debug(f"Closing not existing window {e}")
        event.accept()

    def convertCoord(self,coord, type ="dms"):
        if type == "dms":
            a = Angle(coord, u.deg)
            value = str(a.to_string(unit=u.degree,precision=0, sep=("°", "'" ,"''" )))
        elif type == "hms":
            a = Angle(coord, u.deg)
            value = str(a.to_string(unit=u.hour, precision=0, sep=("h", "m", "s")))
        elif type == "lat":
            a = Latitude(coord, u.deg)
            a.wrap_angle = 90 * u.deg
            value = str(a.to_string(unit=u.degree,precision=0, sep=("°", "'" ,"''" )))
          
        elif type == "long":
            a = Longitude(coord, u.deg)
            a.wrap_angle = 180 * u.deg
            value = str(a.to_string(unit=u.degree, precision=0, sep=("°", "'" ,"''" )))

        return str(value)
  
class MatplotlibWidget(Canvas):
    logger = logging.getLogger(__name__)

    def __init__(
        self, parent=None, title="Title", width=5, height=5, dpi=100, hold=True
    ):
        super(MatplotlibWidget, self).__init__(Figure())

        self.setParent(parent)
        self.figure = Figure(figsize=(width, height), dpi=dpi)
        self.figure.patch.set_facecolor("xkcd:black")

        self.canvas = Canvas(self.figure)

    def plot(self):
        self.figure.clear()

        self.axes = self.figure.add_subplot(1, 1, 1)
        self.figure.tight_layout()
        self.axes.axis("off")

        try:
            image = fits.getdata(self.fileName, ext=0)
            im = self.axes.imshow(image, cmap="gray", norm=LogNorm())
        except Exception as e:
            self.logger.error(e)
        self.canvas.draw()

    def setFileName(self, fileName):
        self.fileName = fileName



import sys
import pyqtgraph as pg
import datetime
import time
import numpy as np
import logging

from PyQt5.QtWidgets import QDialog
from .gui.chartWindowGui import *
from .pg_time_axis import DateAxisItem

"""
Charts are plotted using pyqtgrpah library.
Data are read directly from the image list 
model (imageListModel) so charts plot exactly 
what is shown in the image list table view.
Improvements needed:
- Filter colors should be moved in Settings
so that users can customize them.
- A factory to build plots, they are now 
build directly.
"""


class ChartWindow(QDialog):
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        super().__init__()
        self.ui = Ui_Dialog2()
        self.ui.setupUi(self)
        self.setWindowTitle("Charts")
        self.app = app

    def closeEvent(self, event):
        self.app.settings.setValue("sizeChartW", self.size())
        self.app.settings.setValue("posChartW", self.pos())
        try:
            self.close()
        except Exception as e:
            self.logger.debug(f"Closing not existing window {e}")
        event.accept()

    def plot(self, imageListModel):
        self.imageListModel = imageListModel
        try:
            self.resize(self.app.settings.value("sizeChartW"))
            self.move(self.app.settings.value("posChartW"))
        except Exception as e:
            self.logger.error(f"{e}")

        self.show()
        self.ui.labelColorL.setStyleSheet("color:rgb(244, 244, 244);font-weight:bold")
        self.ui.labelColorR.setStyleSheet("color:rgb(255,0,0);font-weight:bold")
        self.ui.labelColorG.setStyleSheet("color:rgb(0, 140, 55);font-weight:bold")
        self.ui.labelColorB.setStyleSheet("color:rgb(0,0,255);font-weight:bold")
        self.ui.labelColorHa.setStyleSheet("color:rgb(190, 255, 0);font-weight:bold")
        self.ui.labelColorOiii.setStyleSheet(
            "color:rgb(150, 200, 255);font-weight:bold"
        )
        self.ui.labelColorSii.setStyleSheet("color:rgb(255, 120, 190);font-weight:bold")
        self.ui.labelColorN.setStyleSheet("color:rgb(120,120,120);font-weight:bold")

        frame = []
        filters = []
        alt = []
        az = []
        fwhm = []
        eccentricity = []
        noise = []
        snrweight = []
        datetimestr = []
        timestampObj = []
        colors = []
        g1 = []
        g2 = []
        g3 = []
        g4 = []
        g5 = []
        g6 = []
        g7 = []
        g8 = []
        g9 = []
        g10 = []
        g11 = []
        g12 = []
        g13 = []
        g14 = []
        g15 = []

        for row in range(imageListModel.rowCount()):
            indexFilters = imageListModel.index(row, 5)
            indexAlt = imageListModel.index(row, 14)
            indexAz = imageListModel.index(row, 15)
            indexDatetimestr = imageListModel.index(row, 16)
            indexFwhm = imageListModel.index(row, 25)
            indexEccentricity = imageListModel.index(row, 26)
            indexSnrweight = imageListModel.index(row, 27)
            indexNoise = imageListModel.index(row, 28)

            filters.append((str(imageListModel.data(indexFilters))))
            alt.append((float(imageListModel.data(indexAlt))))
            az.append((float(imageListModel.data(indexAz))))
            datetimestr.append((str(imageListModel.data(indexDatetimestr))))
            fwhm.append((float(imageListModel.data(indexFwhm))))
            eccentricity.append((float(imageListModel.data(indexEccentricity))))
            snrweight.append((float(imageListModel.data(indexSnrweight))))
            noise.append((float(imageListModel.data(indexNoise))))

            # pg only works with timestamps
            date_time_obj = datetime.datetime.strptime(
                datetimestr[row], "%Y-%m-%dT%H:%M:%S"
            )
            timestampObj.append(datetime.datetime.timestamp(date_time_obj))

            # colors

            if filters[row] in self.app.confFilters["L"]:
                colors.append(pg.mkBrush(244, 244, 244, 255))
            elif filters[row] in self.app.confFilters["R"]:
                colors.append(pg.mkBrush(255, 0, 0, 255))
            elif filters[row] in self.app.confFilters["B"]:
                colors.append(pg.mkBrush(0, 0, 255, 255))
            elif filters[row] in self.app.confFilters["G"]:
                colors.append(pg.mkBrush(0, 140, 55, 255))
            elif filters[row] in self.app.confFilters["Ha"]:
                colors.append(pg.mkBrush(190, 255, 0, 255))
            elif filters[row] in self.app.confFilters["Oiii"]:
                colors.append(pg.mkBrush(150, 200, 255, 255))
            elif filters[row] in self.app.confFilters["Sii"]:
                colors.append(pg.mkBrush(255, 120, 190, 255))
            else:
                colors.append(pg.mkBrush(120, 120, 120, 255))

            # create data sets
            g1.append({"pos": (alt[row], az[row]), "brush": colors[row]})
            g2.append({"pos": (az[row], alt[row]), "brush": colors[row]})
            g3.append({"pos": (timestampObj[row], alt[row]), "brush": colors[row]})
            g4.append({"pos": (alt[row], fwhm[row]), "brush": colors[row]})
            g5.append({"pos": (az[row], fwhm[row]), "brush": colors[row]})
            g6.append({"pos": (timestampObj[row], fwhm[row]), "brush": colors[row]})
            g7.append({"pos": (alt[row], eccentricity[row]), "brush": colors[row]})
            g8.append({"pos": (az[row], eccentricity[row]), "brush": colors[row]})
            g9.append(
                {"pos": (timestampObj[row], eccentricity[row]), "brush": colors[row]}
            )
            g10.append({"pos": (alt[row], noise[row]), "brush": colors[row]})
            g11.append({"pos": (az[row], noise[row]), "brush": colors[row]})
            g12.append({"pos": (timestampObj[row], noise[row]), "brush": colors[row]})
            g13.append({"pos": (alt[row], snrweight[row]), "brush": colors[row]})
            g14.append({"pos": (az[row], snrweight[row]), "brush": colors[row]})
            g15.append(
                {"pos": (timestampObj[row], snrweight[row]), "brush": colors[row]}
            )

        pg.setConfigOption("background", "k")
        pg.setConfigOption("foreground", "w")
        pg.setConfigOption("antialias", True)
        pg.setConfigOptions(imageAxisOrder="row-major")
        print(g3)
        # Graph1: Alt-Az
        self.ui.graphWidget1.setLabel("left", "Az (deg)", color="white", size=30)
        self.ui.graphWidget1.setLabel("bottom", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget1.showGrid(x=True, y=True)

        scatter1 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget1.addItem(scatter1)
        scatter1.setData(g1)
        self.lrAlt = pg.LinearRegionItem([100, 200])
        self.lrAlt.setZValue(-10)
        self.ui.graphWidget1.addItem(self.lrAlt)

        # Graph2: AZ-Alt
        self.ui.graphWidget2.setLabel("left", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget2.setLabel("bottom", "Az (deg)", color="white", size=30)
        self.ui.graphWidget2.showGrid(x=True, y=True)

        scatter2 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget2.addItem(scatter2)
        scatter2.setData(g2)
        self.lrAz = pg.LinearRegionItem([100, 200])
        self.lrAz.setZValue(-10)
        self.ui.graphWidget2.addItem(self.lrAz)

        # Graph3: Time-Alt
        self.ui.graphWidget3.setLabel("left", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget3.showGrid(x=True, y=True)

        axis3 = DateAxisItem(orientation="bottom")
        axis3.attachToPlotItem(self.ui.graphWidget3.getPlotItem())
        axis3.setLabel("Time", units="h")

        scatter3 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget3.addItem(scatter3)
        scatter3.setData(g3)
        self.lrDate = pg.LinearRegionItem([min(timestampObj), max(timestampObj)])
        self.lrDate.setZValue(-10)
        self.ui.graphWidget3.addItem(self.lrDate)

        # Graph4: Alt-FWHM
        self.ui.graphWidget4.setLabel("bottom", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget4.setLabel("left", "FWHM", color="white", size=30)
        self.ui.graphWidget4.showGrid(x=True, y=True)

        scatter4 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget4.addItem(scatter4)
        scatter4.setData(g4)

        # Graph5: Az-FWHM
        self.ui.graphWidget5.setLabel("bottom", "Az (deg)", color="white", size=30)
        self.ui.graphWidget5.setLabel("left", "FWHM", color="white", size=30)
        self.ui.graphWidget5.showGrid(x=True, y=True)

        scatter5 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget5.addItem(scatter5)
        scatter5.setData(g5)

        # Graph6: Time-FWHM
        self.ui.graphWidget6.setLabel("left", "FWHM", color="white", size=30)
        self.ui.graphWidget6.showGrid(x=True, y=True)

        axis6 = DateAxisItem(orientation="bottom")
        axis6.attachToPlotItem(self.ui.graphWidget6.getPlotItem())
        axis6.setLabel("Time", units="h")

        scatter6 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget6.addItem(scatter6)
        scatter6.setData(g6)

        # Graph7: Alt-Eccentricity
        self.ui.graphWidget7.setLabel("bottom", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget7.setLabel("left", "Eccentricity", color="white", size=30)
        self.ui.graphWidget7.showGrid(x=True, y=True)

        scatter7 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget7.addItem(scatter7)
        scatter7.setData(g7)

        # Graph8: Az-Eccentricity
        self.ui.graphWidget8.setLabel("bottom", "Az (deg)", color="white", size=30)
        self.ui.graphWidget8.setLabel("left", "Eccentricity", color="white", size=30)
        self.ui.graphWidget8.showGrid(x=True, y=True)

        scatter8 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget8.addItem(scatter8)
        scatter8.setData(g8)

        # Graph9: Time-Eccentricity
        self.ui.graphWidget9.setLabel("left", "Eccentricity", color="white", size=30)
        self.ui.graphWidget9.showGrid(x=True, y=True)

        axis9 = DateAxisItem(orientation="bottom")
        axis9.attachToPlotItem(self.ui.graphWidget9.getPlotItem())
        axis9.setLabel("Time", units="h")

        scatter9 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget9.addItem(scatter9)
        scatter9.setData(g9)

        # Graph10: Alt-Noise
        self.ui.graphWidget10.setLabel("bottom", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget10.setLabel("left", "Noise", color="white", size=30)
        self.ui.graphWidget10.showGrid(x=True, y=True)

        scatter10 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget10.addItem(scatter10)
        scatter10.setData(g10)

        # Graph11: Az-Noise
        self.ui.graphWidget11.setLabel("bottom", "Az (deg)", color="white", size=30)
        self.ui.graphWidget11.setLabel("left", "Noise", color="white", size=30)
        self.ui.graphWidget11.showGrid(x=True, y=True)

        scatter11 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget11.addItem(scatter11)
        scatter11.setData(g11)

        # Graph12: Time-Noise
        self.ui.graphWidget12.setLabel("left", "Noise", color="white", size=30)
        self.ui.graphWidget12.showGrid(x=True, y=True)

        axis12 = DateAxisItem(orientation="bottom")
        axis12.attachToPlotItem(self.ui.graphWidget12.getPlotItem())
        axis12.setLabel("Time", units="h")

        scatter12 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget12.addItem(scatter12)
        scatter12.setData(g12)

        # Graph13: Alt-SNRWeight
        self.ui.graphWidget13.setLabel("bottom", "Alt (deg)", color="white", size=30)
        self.ui.graphWidget13.setLabel("left", "SNRWeight", color="white", size=30)
        self.ui.graphWidget13.showGrid(x=True, y=True)

        scatter13 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget13.addItem(scatter13)
        scatter13.setData(g13)

        # Graph14: Az-SNRWeight
        self.ui.graphWidget14.setLabel("bottom", "Az (deg)", color="white", size=30)
        self.ui.graphWidget14.setLabel("left", "SNRWeight", color="white", size=30)
        self.ui.graphWidget14.showGrid(x=True, y=True)

        scatter14 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget14.addItem(scatter14)
        scatter14.setData(g14)

        # Graph15: Time-SNRWeight
        self.ui.graphWidget15.setLabel("left", "SNRWeight", color="white", size=30)
        self.ui.graphWidget15.showGrid(x=True, y=True)

        axis15 = DateAxisItem(orientation="bottom")
        axis15.attachToPlotItem(self.ui.graphWidget15.getPlotItem())
        axis15.setLabel("Time", units="h")

        scatter15 = pg.ScatterPlotItem(brush=pg.mkBrush(width=5, color="w"), symbol="o")
        self.ui.graphWidget15.addItem(scatter15)
        scatter15.setData(g15)

        self.lrAlt.sigRegionChanged.connect(self.updatePlotAlt)
        self.ui.graphWidget4.sigXRangeChanged.connect(self.updateRegionAlt)
        self.ui.graphWidget7.sigXRangeChanged.connect(self.updateRegionAlt)
        self.ui.graphWidget10.sigXRangeChanged.connect(self.updateRegionAlt)
        self.ui.graphWidget13.sigXRangeChanged.connect(self.updateRegionAlt)

        self.lrAz.sigRegionChanged.connect(self.updatePlotAz)
        self.ui.graphWidget5.sigXRangeChanged.connect(self.updateRegionAz)
        self.ui.graphWidget8.sigXRangeChanged.connect(self.updateRegionAz)
        self.ui.graphWidget11.sigXRangeChanged.connect(self.updateRegionAz)
        self.ui.graphWidget14.sigXRangeChanged.connect(self.updateRegionAz)

        self.lrDate.sigRegionChanged.connect(self.updatePlotDate)
        self.ui.graphWidget6.sigXRangeChanged.connect(self.updateRegionDate)
        self.ui.graphWidget9.sigXRangeChanged.connect(self.updateRegionDate)
        self.ui.graphWidget12.sigXRangeChanged.connect(self.updateRegionDate)
        self.ui.graphWidget15.sigXRangeChanged.connect(self.updateRegionDate)

    def updatePlotAlt(self):
        self.ui.graphWidget4.setXRange(*self.lrAlt.getRegion(), padding=0)
        self.ui.graphWidget7.setXRange(*self.lrAlt.getRegion(), padding=0)
        self.ui.graphWidget10.setXRange(*self.lrAlt.getRegion(), padding=0)
        self.ui.graphWidget13.setXRange(*self.lrAlt.getRegion(), padding=0)

    def updateRegionAlt(self, region):
        self.lrAlt.setRegion(self.ui.graphWidget4.getViewBox().viewRange()[0])

    def updatePlotAz(self):
        self.ui.graphWidget5.setXRange(*self.lrAz.getRegion(), padding=0)
        self.ui.graphWidget8.setXRange(*self.lrAz.getRegion(), padding=0)
        self.ui.graphWidget11.setXRange(*self.lrAz.getRegion(), padding=0)
        self.ui.graphWidget14.setXRange(*self.lrAz.getRegion(), padding=0)

    def updateRegionAz(self, region):
        self.lrAz.setRegion(self.ui.graphWidget5.getViewBox().viewRange()[0])

    def updatePlotDate(self):
        self.ui.graphWidget6.setXRange(*self.lrDate.getRegion(), padding=0)
        self.ui.graphWidget9.setXRange(*self.lrDate.getRegion(), padding=0)
        self.ui.graphWidget12.setXRange(*self.lrDate.getRegion(), padding=0)
        self.ui.graphWidget15.setXRange(*self.lrDate.getRegion(), padding=0)

    def updateRegionDate(self):
        self.lrDate.setRegion(self.ui.graphWidget6.getViewBox().viewRange()[0])



import sys
import os
import csv
import hashlib
import ntpath
import logging
import re

from pathlib import Path
from os.path import expanduser
from astropy.io import fits
from .astropyCalc import AstropyCalc
from PyQt5 import QtWidgets
from PyQt5 import QtSql, QtGui, QtCore
from datetime import datetime

from .importTableModel import *

"""
This class manages imports: first step is to import
FITS files and their headers scanning dirs. Then we
can add information like FWHM, noise etc importing
a CSV file as the output from SubFrameSelector from
Pixinsight. There are read and save method for both
operations (FITS import and CSV import).
The models (both FITS and CSV) are managed by 
ImportTableModel.
"""


class ImportFitsTab:
    logger = logging.getLogger(__name__)

    def __init__(self, mainW, app):
        super().__init__()
        self.mainW = mainW
        self.app = app

        # GUI logging handler
        self.handler = h = QtHandler(self.printLogMessage)
        fs = "%(asctime)s  %(levelname)-8s %(message)s"
        formatter = logging.Formatter(fs)
        h.setFormatter(formatter)
        h.setLevel(logging.INFO)
        self.logger.addHandler(h)

    # Slot used by ImportDir parser class
    def addResultsToModel(self, row):
        self._data.append(row)
        self.model.layoutChanged.emit()

    # Slot for printing log message in the main thread
    def printLogMessage(self, msg, record):
        color = self.app.COLORS.get(record.levelno, "black")
        s = '<pre><font color="%s">%s</font></pre>' % (color, msg)
        self.mainW.ui.plainTextEditLogFits.appendHtml(s)

    # Directory parser (main thread)
    def importFitsDir(self):

        # Choose dir and filter .fits files
        input_dir = QtWidgets.QFileDialog.getExistingDirectory(
            None, "Select a folder:", expanduser("~")
        )
        fileList = list(Path(input_dir).rglob("*.[Ff][Ii][Tt]*"))

        self.mainW.ui.lineEditFitsDir.setText(input_dir)
        self.mainW.importDir.set_file(fileList)
        self._headers = self.app.filterDictToList("fitsHeader", "description")
        self._data = []

        # Populate tableview
        self.model = ImportTableModel(self._data, self._headers)
        self.mainW.ui.tableViewImportFits.setSortingEnabled(True)
        self.mainW.ui.tableViewImportFits.setAlternatingRowColors(True)
        self.mainW.ui.tableViewImportFits.setModel(self.model)

    def deleteRows(self):

        indexes = self.mainW.ui.tableViewImportFits.selectedIndexes()
        countRows = self.mainW.ui.tableViewImportFits.selectionModel().selectedRows()
        if indexes:
            ret = QtWidgets.QMessageBox.question(None,
                                       "Delete rows",
                                       "Are you sure to delete selected rows?",
                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No
                                       )

            if ret == QtWidgets.QMessageBox.Yes:
                self.model.removeRows(indexes[0].row(), countRows, None)
                
    # Saves each row in the tableview into the database
    def saveFits(self):

        separator = ","
        fieldInsert = self.app.filterDictToList("fitsHeader", "keys")
        fitsDefault = self.app.filterDictToList("fitsDefault", "keys")
        sqlInsert = separator.join(fieldInsert)

        rows = self.model.rowCount(self.mainW.ui.tableViewImportFits.rootIndex())
        targetOverride = self.mainW.ui.lineEditOverrideTarget.text()
        self.logger.info(f"Start saving {rows} files")
        for row in range(rows):
            emptyCellCheck = True
            query = "INSERT INTO images ( " + sqlInsert + ") VALUES("

            for col, val in enumerate(fieldInsert):
                currentIndex = self.model.index(row, col)
                item = self.model.data(currentIndex, QtCore.Qt.DisplayRole)
                if len(str(item)) > 0:
                    # Override target name if asked to
                    if col == 2 and targetOverride:
                        item = targetOverride
                        self.model.setData(
                            self.model.index(row, col), item, QtCore.Qt.EditRole
                        )
                    query += "'" + str(item) + "',"
                    if col == 0:
                        fileName = str(item)
                else:
                    # if no value is read in FIT header try default value
                    if val in fitsDefault:
                        self.logger.error(
                            "Should not be in here because default values are already set"
                        )
                        item = self.app.conf[val]["fitsDefault"]
                        query += "'" + str(item) + "',"
                    else:
                        emptyCellCheck = False

            if emptyCellCheck == False:
                self.logger.error(f"File not inserted: {fileName}")
                continue
            query = query[:-1]
            query += ");"
            try:
                ret = self.app.db.exec(query)
                if ret.lastError().number() == 19:
                    self.model.setData(
                        self.model.index(row, 1),
                        "Error: FITS file already exists in database",
                        QtCore.Qt.EditRole,
                    )
                    error = ret.lastError().text()
                    self.logger.error(f"{error}")
                    self.logger.error(f"{query}")
                elif ret.lastError().number() > 0:
                    error = ret.lastError().text()
                    self.logger.error(f"{error}")
                    self.logger.error(f"{query}")
                else:
                    self.model.setData(
                        self.model.index(row, 1),
                        "OK: FITS file saved",
                        QtCore.Qt.EditRole,
                    )
                    self.logger.info(f"Saved file {fileName}")
                    self.logger.debug(f"FITS file saved with query {query}")

            except Exception as e:
                self.logger.error(f"Insert error {e}")

            self.model.layoutChanged.emit()
        self.logger.info(f"Saving completed")
        # Force image list reload data
        self.mainW.imageSourceModel.select()
        while self.mainW.imageSourceModel.canFetchMore():
            self.mainW.imageSourceModel.fetchMore()
        self.mainW.filterRegExpChanged()


"""
ImportDir is the thread reading FITS files. Should be
moved to a separate files but is kept here just for
convenience. Thread is created in mainWindow.py
"""


class ImportDir(QtCore.QObject):
    logger = logging.getLogger(__name__)

    # notifies main thread when a full row is read
    match_found = QtCore.pyqtSignal(list)
    # notifies when import finished
    finished = QtCore.pyqtSignal()

    def __init__(self, app):
        super().__init__()
        self.app = app

    def set_file(self, fileList):
        self.fileList = fileList

    @qtc.pyqtSlot()
    def worker(self):
        self._worker(self.fileList)
        self.finished.emit()

    def _worker(self, fileList):

        dbFitsParameters = {}
        fitsKeyList = self.app.filterDictToList("fitsHeader", "keys")
        fitsDefault = self.app.filterDictToList("fitsDefault", "keys")

        for i, f in enumerate(fileList):
            self.logger.info(f"{i+1}-Start reading file {f}")
            row = []

            # Parse FITS header for given file
            fitsFileParameters = self.parseFitsHeader(f)

            # Create a row with all requested fits header values from file
            for dbFitsParameters in fitsKeyList:
                oneitem = str(
                    fitsFileParameters[self.app.conf[dbFitsParameters]["fitsHeader"]]
                )
                row.insert(self.app.conf[dbFitsParameters]["order"], oneitem)

            # Then some values in the row are added or overwritten
            row[fitsKeyList.index("file")] = str(f)
            row[fitsKeyList.index("hash")] = self.hashFile(f)

            # RA/DEC coord conversion
            AstropyCalcObj = AstropyCalc(self.app)
            c = AstropyCalcObj.coordConversion(
                row[fitsKeyList.index("ra")], row[fitsKeyList.index("dec")]
            )
            if c:
                row[fitsKeyList.index("ra")] = str(round(c.ra.degree, self.app.config["precision"]))
                row[fitsKeyList.index("dec")] = str(round(c.dec.degree, self.app.config["precision"]))

            # Strip subseconds
            dateobs = row[fitsKeyList.index("date")]
            dateobs = dateobs.split(".")[0]
            row[fitsKeyList.index("date")] = dateobs

            # Calculate Alt, Az coordinates
            strAlt = "0"
            strAz = "0"

            s = AstropyCalcObj.longLatConversion(
                row[fitsKeyList.index("sitelong")], row[fitsKeyList.index("sitelat")]
            )
            if s and c:
                row[fitsKeyList.index("sitelong")] = str(round(s.ra.degree, self.app.config["precision"]))
                row[fitsKeyList.index("sitelat")] = str(round(s.dec.degree, self.app.config["precision"]))
                altAz = AstropyCalcObj.getAltAzCoord(
                    c.ra.degree,
                    c.dec.degree,
                    row[fitsKeyList.index("date")],
                    s.ra.degree,
                    s.dec.degree,
                )
                if altAz:
                    strAlt = str("{0.alt:.4f}".format(altAz))[:-4]
                    strAz = str("{0.az:.4f}".format(altAz))[:-4]

            row[fitsKeyList.index("alt")] = strAlt
            row[fitsKeyList.index("az")] = strAz

            # if no value is read in FIT header try default value
            for keyD, valD in enumerate(fitsDefault):

                if len(row[fitsKeyList.index(valD)]) == 0:
                    defaultValue = self.app.conf[valD]["fitsDefault"]
                    defaultField = self.app.conf[valD]["description"]
                    self.logger.warning(
                        f"Inserted default value {defaultValue} for {defaultField} keyword"
                    )
                    row[fitsKeyList.index(valD)] = defaultValue

            # check for null value that prevents db insert
            for k, v in enumerate(row):
                if not v:
                    self.logger.error(
                        f"Null value for {fitsKeyList[k]} keyword: in Settings "
                        "fix keyword name or set a default value else the file "
                        "cannot be inserted into the database"
                    )

            self.logger.debug(row)
            self.logger.info(f"{i+1}-End reading file {f}")
            self.match_found.emit(row)

    def parseFitsHeader(self, fitsFile):
        returnParameter = {}
        hdu = fits.open(fitsFile)
        hdu.verify("silentfix")
        hdr = hdu[0].header
        for param in self.app.filterDictToList("fitsHeader"):
            if param in hdr:
                returnParameter[param] = hdr[param]
            else:
                returnParameter[param] = ""
        return returnParameter

    def hashFile(self, fileName):
        with open(fileName, "rb") as afile:
            hasher = hashlib.md5()
            buf = afile.read(self.app.BLOCKSIZE)
            for i in range(5):
                hasher.update(buf)
                buf = afile.read(self.app.BLOCKSIZE)
        hash = hasher.hexdigest()
        return hash


"""
This two classes manage the GUI log messages using an additional 
log handler. Be careful that log messages are logged from outside 
of the main thread and widgets are not thread safe so we cannot 
update the GUI from inside the worker but build a signal/slot for
that. 
"""


class Signaller(QtCore.QObject):
    signal = QtCore.pyqtSignal(str, logging.LogRecord)


class QtHandler(logging.Handler):
    def __init__(self, slotfunc, *args, **kwargs):
        super(QtHandler, self).__init__(*args, **kwargs)
        self.signaller = Signaller()
        self.signaller.signal.connect(slotfunc)

    def emit(self, record):
        s = self.format(record)
        self.signaller.signal.emit(s, record)



# -----------------------------------------------------------
# Astronomy Domine:
#
# (C) 2020 Ferrante Enriques, ferrante.enriques@gmail.com
# Released under MIT License
# -----------------------------------------------------------

import sys
import os
import json
import logging
import codecs
import shutil
from PyQt5 import QtSql
from PyQt5 import QtWidgets as qtw

from PyQt5.QtCore import QSettings
from datetime import date

from .mainWindow import *

"""
AstroDom application class. It inizializes logs,
sets up database connection, reads config files,
eventually creates a new db, defines some useful
methods and calls the mainWindow GUI. In children
classes it is referenced as 'app'.
"""


class AstroDom:
    astrodomDir = os.getcwd()
    directory = os.path.dirname(__file__)
    
    #if config files don't exist copy the defaults
    configFile = os.path.join(directory, "config", "config.json")
    if not os.path.isfile(configFile):
        shutil.copyfile(os.path.join(directory, "config", "config-default.json"), configFile)
    with open(configFile, "r") as config:
        config = json.load(config)

    logDir = os.path.join(directory, "logs")
    if not os.path.exists(logDir):
        os.makedirs(logDir)

    logging.basicConfig(
        filename=os.path.join(logDir, str(date.today()) + ".log"),
        level=config["debug"],
        datefmt="%Y-%m-%d %H:%M:%S",
        format="%(asctime)s %(name)s - %(levelname)s - %(message)s",
    )
    logger = logging.getLogger(__name__)

    def __init__(self):

        # Read configurations from json file
        profileFile = "profile-" + self.config["profile"] + ".json"
        with open(
            os.path.join(self.directory, "config", profileFile), "r"
        ) as configFields:
            self.conf = json.load(configFields)
            self.logger.debug("Read Config Fits File " + str(configFields))

        with open(
            os.path.join(self.directory, "config", "configFilters.json"), "r"
        ) as configFileFilters:
            self.confFilters = json.load(configFileFilters)
            self.logger.debug("Read Config Filters File " +
                              str(configFileFilters))

        self.settings = QSettings("fenriques", "AstroDom")

        # For hashing files
        self.BLOCKSIZE = 1024768

        self.DBDRIVER = "QSQLITE"
        self.COLORS = {
            logging.DEBUG: "black",
            logging.INFO: "#4af",
            logging.WARNING: "orange",
            logging.ERROR: "red",
            logging.CRITICAL: "purple",
        }
        self.version = self.get_version("__init__.py")
        # Create db connection
        if not self.createConnection():
            QMessageBox.about(None, "Message", "Cannot connect to database.")
            exit(0)
        self.setUpDbStructure()
        self.mainWindow = MainWindow(self)

    # Mainly used to extract a list from configurations
    def filterDictToList(self, filter, returnDifferentParameter=None):
        returnList = []
        for key, dictInfo in self.conf.items():
            for subKey in dictInfo:
                if subKey == filter and dictInfo[subKey] != "":
                    if returnDifferentParameter:
                        if returnDifferentParameter == "keys":
                            returnList.append(key)
                        else:
                            returnList.append(
                                dictInfo[returnDifferentParameter])
                    else:
                        returnList.append(dictInfo[subKey])
        return returnList

    def createConnection(self):
        # If there's no database, create one.
        if len(self.config["dbname"]) == 0:
            dbName, ok = DbDialog.getDataBaseName()
            if not ok or len(dbName) == 0:
                return False
            self.config["dbname"] = dbName
            with open(
                os.path.join(self.directory, "config", "config.json"), "w"
            ) as outfile3:
                json.dump(self.config, outfile3)
                self.logger.debug(self.config)

        # Establish connection
        self.logger.info("Set up db connection " + self.DBDRIVER)
        self.db = QtSql.QSqlDatabase.addDatabase(self.DBDRIVER)
        self.db.setDatabaseName(
            os.path.join(self.directory, "config",
                         self.config["dbname"] + ".db")
        )
        if not self.db.open():
            self.logger.critical("Failed to set up db connection")
            return False
        return True

    # the database structure is stored in a json config file
    def setUpDbStructure(self):
        separator = ","
        sqlString = separator.join(self.filterDictToList("fieldType"))
        tableCreate = (
            "CREATE TABLE IF NOT EXISTS 'images' ("
            + "[imageId] INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,"
            + sqlString
            + ")"
        )
        self.logger.debug("Try to create table: " + tableCreate)
        ret = self.db.exec(tableCreate)
        if ret.lastError().number():
            self.logger.error(ret.lastError().number())
            self.logger.error(ret.lastError().text())

        self.logger.debug("Try to create index")
        ret = self.db.exec(
            "CREATE UNIQUE INDEX idx_images_hash ON images (hash);")
        if ret.lastError().text():
            self.logger.warning(str(ret.lastError().text()))

        self.logger.info("DB setup complete")

    def read(self, rel_path):
        here = os.path.abspath(os.path.dirname(__file__))
        with codecs.open(os.path.join(here, rel_path), "r") as fp:
            return fp.read()

    def get_version(self, rel_path):
        for line in self.read(rel_path).splitlines():
            if line.startswith("__version__"):
                # __version__ = "0.9"
                delim = '"' if '"' in line else "'"
                return line.split(delim)[1]
        else:
            raise RuntimeError("Unable to find version string.")


class DbDialog(QDialog):
    def __init__(self, parent=None):
        super(DbDialog, self).__init__(parent)

        layout = QtWidgets.QVBoxLayout(self)
        self.setWindowTitle("Message")
        self.labelMsg = QtWidgets.QLabel()
        self.labelMsg.setText(
            "Please choose a name for the storage.\nFor example 'MyAstroImages', 'Images6-2020', 'IC434' ")
        layout.addWidget(self.labelMsg)
        self.lineEditDbName = QtWidgets.QLineEdit()
        layout.addWidget(self.lineEditDbName)

        # OK and Cancel buttons
        buttons = QtWidgets.QDialogButtonBox(
            QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel,
            Qt.Horizontal, self)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

    # get current date and time from the dialog
    def dbName(self):
        return self.lineEditDbName.text()

    # static method to create the dialog and return (date, time, accepted)
    @staticmethod
    def getDataBaseName(parent=None):
        dialog = DbDialog(parent)
        result = dialog.exec_()
        dbName = dialog.dbName()
        return (dbName, result == QDialog.Accepted)




import sys
import os
import csv
import hashlib
import ntpath
import logging

from pathlib import Path,PureWindowsPath,PurePosixPath, PurePath
from PyQt5 import QtWidgets
from PyQt5 import QtSql, QtGui, QtCore

from .importTableModel import *

'''
This class manages imports: first step is to import
FITS files and their headers scanning dirs. Then we
can add information like FWHM, noise etc importing
a CSV file as the output from SubFrameSelector from
Pixinsight. There are read and save method for both
operations (FITS import and CSV import).
The models (both FITS and CSV) are managed by 
ImportTableModel.
'''
class ImportCsvTab():
    logger = logging.getLogger(__name__)

    def __init__(self, mainW, app):
        super().__init__()
        self.mainW = mainW
        self.app = app
        # GUI logging handler
        self.handler = h = QtHandler(self.printLogMessage)
        fs = "%(asctime)s  %(levelname)-8s %(message)s"
        formatter = logging.Formatter(fs)
        h.setFormatter(formatter)
        h.setLevel(logging.INFO)
        self.logger.addHandler(h)
    
    # Slot for printing log message in the main thread
    def printLogMessage(self, msg, record):
        color = self.app.COLORS.get(record.levelno, "black")
        s = '<pre><font color="%s">%s</font></pre>' % (color, msg)
        self.mainW.ui.plainTextEditLogCsv.appendHtml(s)   
    
    def importCsvFile(self):
        filename, _ = QtWidgets.QFileDialog.getOpenFileName(
            self.mainW,
            'Select a CSV file to open…',
            QtCore.QDir.homePath(),
            'CSV Files (*.csv) ;; All Files (*)'
        )
        if filename:
            with open(filename) as fh:
                csvreader = csv.reader(fh)
                dataTemp = list(csvreader)
            self.mainW.ui.lineEditCsv.setText(filename)
            self.logger.info(f"Reading {filename}")
        else:
            return
        csvList = self.app.filterDictToList('pix_csv')

        # First rows of the CSV file don't contain images.
        checkCsv = False
        self._data = []
        self._headers = []
        i = 1
        for row, val in enumerate(dataTemp):
            if dataTemp[row][0] == 'Subframe Scale':
                subframeScale = str(dataTemp[row][1])
            if dataTemp[row][0] == 'Camera Gain':
                cameraGain = str(dataTemp[row][1])
            if dataTemp[row][0] == 'Camera Resolution':
                cameraResolution = str(dataTemp[row][1])
            if dataTemp[row][0] == 'Scale Unit':
                scaleUnit = str(dataTemp[row][1])
            if dataTemp[row][0] == 'Data Unit':
                dataUnit = str(dataTemp[row][1])

            if checkCsv == True:
                dataTemp[row].insert(0, subframeScale)
                dataTemp[row].insert(1, cameraGain)
                dataTemp[row].insert(2, cameraResolution)
                dataTemp[row].insert(3, scaleUnit)
                dataTemp[row].insert(4, dataUnit)

                self.logger.info(f"Row n {i}")
                i+=1
                # Items (columns) for each row
                filteredRow = []
                for col in range(len(val)):
                    if str(col) in csvList:
                        item = dataTemp[row][col]
                        # Check if the file (hash) exists in the database
                        if col == 8:
                            filenameMatch = ntpath.splitext(
                                ntpath.basename(item))[0]
                            '''
                            TO BE COMPLETED:
                            filenameMatch = ntpath.splitext(
                                ntpath.basename(item))[0]
                            sqlStatementF = "SELECT file FROM images where file like '%"+filenameMatch+"'%"

                            rF = self.app.db.exec(sqlStatementF)
                            rF.next()
                            if rF.value(0):
                                print("trovato")
                            else:
                                print("FITS file not found")
                            '''
                            hashItem = self.hashFile(item)
                            pathFrom =self.mainW.ui.lineEditPathConversionFrom.text()
                            pathTo = self.mainW.ui.lineEditPathConversionTo.text()
                            if len(pathFrom)>0:
                                pathFrom=pathFrom.replace('\\','/')
                                pathTo=pathTo.replace('\\','/')
                                item = item.replace(pathFrom, pathTo)
                                item =str(PurePath(item))
                                           
                            hashItem = self.hashFile(item)
                            sqlStatement = "SELECT hash FROM images where hash = '"+hashItem+"'"

                            r = self.app.db.exec(sqlStatement)
                            r.next()
                            if r.value(0):
                                strStatus = "File found"
                                self.logger.info(
                                    f"File {filenameMatch} found in the database")
                            else:
                                strStatus = "File not found"
                                self.logger.error(
                                    f"File {filenameMatch} not found in the database")

                        filteredRow.insert(col, str(item))
                
                # Append info about matching fits file found/not found
                filteredRow.insert(len(filteredRow), strStatus)
                self._data.append(filteredRow)

            # Headers row are read after 'index' in csv file.
            if dataTemp[row][0] == 'Index':
                self._headers = self.app.filterDictToList(
                    'pix_csv', 'description')
                self._headers.append('Status')
                checkCsv = True
                
        if checkCsv == True:
            self.model = ImportTableModel(self._data, self._headers)
            self.mainW.ui.tableViewImportCsv.setModel(self.model)
            self.mainW.ui.tableViewImportCsv.setSortingEnabled(False)
        else:
            self.logger.error("Invalid CSV format")

    def deleteRows(self):
        indexes = self.mainW.ui.tableViewImportCsv.selectedIndexes()
        countRows = self.mainW.ui.tableViewImportCsv.selectionModel().selectedRows()
        if indexes:
            ret = QtWidgets.QMessageBox.question(None,
                                       "Delete rows",
                                       "Are you sure to delete selected rows?",
                                       QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No
                                       )

            if ret == QtWidgets.QMessageBox.Yes:
                self.model.removeRows(indexes[0].row(), countRows, None)

    def hashFile(self, fileName):
        try:
            fileName = PurePath(fileName)
        
            with open(fileName, 'rb') as afile:
                hasher = hashlib.md5()
                buf = afile.read(self.app.BLOCKSIZE)
                for i in range(5):
                    hasher.update(buf)
                    buf = afile.read(self.app.BLOCKSIZE)
            hash = hasher.hexdigest()
            return hash
        except Exception as e:
            self.logger.error(f"Hash file: fits not found:  {fileName}")
        return ""

    # Data from PI csv (FWHM, Noise etc) are imported in db
    def saveCsv(self):

        fieldUpdate = self.app.filterDictToList('pix_csv', 'keys')
        rows = self.model.rowCount(self.mainW.ui.tableViewImportCsv.rootIndex())
        self.logger.info(f"Saving {rows} CSV rows")
        
        # Each row from table view is an SQL update statement
        for row in range(rows):
            query = "UPDATE images SET "
            csvHash = ''
            for col in range(len(fieldUpdate)):
                currentIndex = self.model.index(row, col)
                item = self.model.data(currentIndex, QtCore.Qt.DisplayRole)
                if item is not None:
                    query += str(fieldUpdate[col]) + "= '"+str(item)+"',"
                    if fieldUpdate[col] == "csvFile":
                        fileName = str(item)
                        csvHash = self.hashFile(item)
                else:
                    query += "'',"
            query = query[:-1]
            query += " WHERE hash = '"+csvHash + "';"

            try:
                ret = self.app.db.exec(query)
                error = ret.lastError().text()
                if ret.lastError().number() > 0:
                    self.logger.error(f"{error}")
                    self.logger.error(f"{query}")
                else:
                    if ret.numRowsAffected() < 1:
                        self.logger.error(f"{fileName} not found")
                    else:
                        self.model.setData(
                            self.model.index(row, 10), "OK: FITS file updated", QtCore.Qt.EditRole)
                        self.logger.info(f"{fileName} updated")
                        self.logger.debug(
                            f"OK: FITS file updated with query {query}")

            except Exception as e:
                self.logger.error(f"Update error {e}")

            self.model.layoutChanged.emit()
        
        # Force image list data reload
        self.mainW.imageSourceModel.select()
        while (self.mainW.imageSourceModel.canFetchMore()):
            self.mainW.imageSourceModel.fetchMore()
        self.mainW.filterRegExpChanged()


"""
This two classes manage the GUI log messages using an additional 
log handler. Be careful that log messages are logged from outside 
of the main thread and widgets are not thread safe so we cannot 
update the GUI from inside the worker but build a signal/slot for
that. 
"""


class Signaller(QtCore.QObject):
    signal = QtCore.pyqtSignal(str, logging.LogRecord)


class QtHandler(logging.Handler):
    def __init__(self, slotfunc, *args, **kwargs):
        super(QtHandler, self).__init__(*args, **kwargs)
        self.signaller = Signaller()
        self.signaller.signal.connect(slotfunc)

    def emit(self, record):
        s = self.format(record)
        self.signaller.signal.emit(s, record)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'chartWindow.ui'
#
# Created by: PyQt5 UI code generator 5.13.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog2(object):
    def setupUi(self, Dialog2):
        Dialog2.setObjectName("Dialog2")
        Dialog2.resize(1276, 1184)
        self.gridLayout = QtWidgets.QGridLayout(Dialog2)
        self.gridLayout.setObjectName("gridLayout")
        self.graphWidget2 = PlotWidget(Dialog2)
        self.graphWidget2.setObjectName("graphWidget2")
        self.gridLayout.addWidget(self.graphWidget2, 1, 1, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(Dialog2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(50)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMinimumSize(QtCore.QSize(0, 60))
        self.groupBox.setObjectName("groupBox")
        self.labelColorL = QtWidgets.QLabel(self.groupBox)
        self.labelColorL.setGeometry(QtCore.QRect(10, 30, 91, 18))
        self.labelColorL.setObjectName("labelColorL")
        self.labelColorR = QtWidgets.QLabel(self.groupBox)
        self.labelColorR.setGeometry(QtCore.QRect(110, 30, 51, 18))
        self.labelColorR.setObjectName("labelColorR")
        self.labelColorHa = QtWidgets.QLabel(self.groupBox)
        self.labelColorHa.setGeometry(QtCore.QRect(260, 30, 61, 18))
        self.labelColorHa.setObjectName("labelColorHa")
        self.labelColorOiii = QtWidgets.QLabel(self.groupBox)
        self.labelColorOiii.setGeometry(QtCore.QRect(330, 30, 41, 18))
        self.labelColorOiii.setObjectName("labelColorOiii")
        self.labelColorSii = QtWidgets.QLabel(self.groupBox)
        self.labelColorSii.setGeometry(QtCore.QRect(370, 30, 41, 18))
        self.labelColorSii.setObjectName("labelColorSii")
        self.labelColorN = QtWidgets.QLabel(self.groupBox)
        self.labelColorN.setGeometry(QtCore.QRect(400, 30, 81, 18))
        self.labelColorN.setObjectName("labelColorN")
        self.labelColorG = QtWidgets.QLabel(self.groupBox)
        self.labelColorG.setGeometry(QtCore.QRect(150, 30, 51, 18))
        self.labelColorG.setObjectName("labelColorG")
        self.labelColorB = QtWidgets.QLabel(self.groupBox)
        self.labelColorB.setGeometry(QtCore.QRect(210, 30, 51, 18))
        self.labelColorB.setObjectName("labelColorB")
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 2)
        self.graphWidget9 = PlotWidget(Dialog2)
        self.graphWidget9.setObjectName("graphWidget9")
        self.gridLayout.addWidget(self.graphWidget9, 3, 2, 1, 1)
        self.graphWidget15 = PlotWidget(Dialog2)
        self.graphWidget15.setObjectName("graphWidget15")
        self.gridLayout.addWidget(self.graphWidget15, 5, 2, 1, 1)
        self.graphWidget3 = PlotWidget(Dialog2)
        self.graphWidget3.setObjectName("graphWidget3")
        self.gridLayout.addWidget(self.graphWidget3, 1, 2, 1, 1)
        self.graphWidget7 = PlotWidget(Dialog2)
        self.graphWidget7.setObjectName("graphWidget7")
        self.gridLayout.addWidget(self.graphWidget7, 3, 0, 1, 1)
        self.graphWidget13 = PlotWidget(Dialog2)
        self.graphWidget13.setObjectName("graphWidget13")
        self.gridLayout.addWidget(self.graphWidget13, 5, 0, 1, 1)
        self.graphWidget14 = PlotWidget(Dialog2)
        self.graphWidget14.setObjectName("graphWidget14")
        self.gridLayout.addWidget(self.graphWidget14, 5, 1, 1, 1)
        self.graphWidget12 = PlotWidget(Dialog2)
        self.graphWidget12.setObjectName("graphWidget12")
        self.gridLayout.addWidget(self.graphWidget12, 4, 2, 1, 1)
        self.graphWidget1 = PlotWidget(Dialog2)
        self.graphWidget1.setObjectName("graphWidget1")
        self.gridLayout.addWidget(self.graphWidget1, 1, 0, 1, 1)
        self.graphWidget5 = PlotWidget(Dialog2)
        self.graphWidget5.setObjectName("graphWidget5")
        self.gridLayout.addWidget(self.graphWidget5, 2, 1, 1, 1)
        self.graphWidget4 = PlotWidget(Dialog2)
        self.graphWidget4.setObjectName("graphWidget4")
        self.gridLayout.addWidget(self.graphWidget4, 2, 0, 1, 1)
        self.graphWidget8 = PlotWidget(Dialog2)
        self.graphWidget8.setObjectName("graphWidget8")
        self.gridLayout.addWidget(self.graphWidget8, 3, 1, 1, 1)
        self.graphWidget6 = PlotWidget(Dialog2)
        self.graphWidget6.setObjectName("graphWidget6")
        self.gridLayout.addWidget(self.graphWidget6, 2, 2, 1, 1)
        self.graphWidget11 = PlotWidget(Dialog2)
        self.graphWidget11.setObjectName("graphWidget11")
        self.gridLayout.addWidget(self.graphWidget11, 4, 1, 1, 1)
        self.graphWidget10 = PlotWidget(Dialog2)
        self.graphWidget10.setObjectName("graphWidget10")
        self.gridLayout.addWidget(self.graphWidget10, 4, 0, 1, 1)
        self.graphWidget6.raise_()
        self.graphWidget9.raise_()
        self.graphWidget1.raise_()
        self.graphWidget2.raise_()
        self.graphWidget3.raise_()
        self.graphWidget4.raise_()
        self.graphWidget5.raise_()
        self.graphWidget8.raise_()
        self.graphWidget7.raise_()
        self.graphWidget11.raise_()
        self.graphWidget10.raise_()
        self.graphWidget12.raise_()
        self.graphWidget13.raise_()
        self.graphWidget14.raise_()
        self.graphWidget15.raise_()
        self.groupBox.raise_()

        self.retranslateUi(Dialog2)
        QtCore.QMetaObject.connectSlotsByName(Dialog2)

    def retranslateUi(self, Dialog2):
        _translate = QtCore.QCoreApplication.translate
        Dialog2.setWindowTitle(_translate("Dialog2", "Dialog"))
        self.groupBox.setTitle(_translate("Dialog2", "Filter Colors"))
        self.labelColorL.setText(_translate("Dialog2", "Luminance"))
        self.labelColorR.setText(_translate("Dialog2", "Red"))
        self.labelColorHa.setText(_translate("Dialog2", "Halpha"))
        self.labelColorOiii.setText(_translate("Dialog2", "Oiii"))
        self.labelColorSii.setText(_translate("Dialog2", "Sii"))
        self.labelColorN.setText(_translate("Dialog2", "Other/NA"))
        self.labelColorG.setText(_translate("Dialog2", "Green"))
        self.labelColorB.setText(_translate("Dialog2", "Blue"))
from pyqtgraph import PlotWidget



import sys
import logging
from PyQt5 import QtWidgets as qtw
from PyQt5 import QtCore as qtc
from PyQt5.QtGui import *
import csv

"""
Generic class that extends QAbstractTableModel.
Used in importTab to show and manage data read
from FITS or CSV files.
"""


class ImportTableModel(qtc.QAbstractTableModel):
    logger = logging.getLogger(__name__)

    def __init__(self, data, headers):
        super().__init__()
        self._data = data
        self._headers = headers
        

    def rowCount(self, parent):
        return len(self._data)

    def columnCount(self, parent):
        return len(self._headers)

    def data(self, index, role):
        data = self._data[index.row()][index.column()]

        if role in (qtc.Qt.DisplayRole, qtc.Qt.EditRole):
            return data
        if role == qtc.Qt.BackgroundRole and (
            data == "Error: FITS file already exists in database"
            or data == "File not found"
            or data == ""
        ):
            return QBrush(qtc.Qt.darkRed)
        if role == qtc.Qt.BackgroundRole and (
            data == "OK: FITS file saved"
            or data == "OK: FITS file updated"
            or data == "File found"
        ):
            return QBrush(qtc.Qt.darkGreen)

    # Additional features methods:
    def headerData(self, section, orientation, role):

        if orientation == qtc.Qt.Horizontal and role == qtc.Qt.DisplayRole:
            return self._headers[section]
        else:
            return super().headerData(section, orientation, role)

    def sort(self, column, order):
        self.layoutAboutToBeChanged.emit()  # needs to be emitted before a sort
        self._data.sort(key=lambda x: x[column])
        if order == qtc.Qt.DescendingOrder:
            self._data.reverse()
        self.layoutChanged.emit()  # needs to be emitted after a sort

    # Methods for Read/Write

    def flags(self, index):
        return super().flags(index) | qtc.Qt.ItemIsEditable

    def setData(self, index, value, role):
        if index.isValid() and role == qtc.Qt.EditRole:
            self._data[index.row()][index.column()] = value
            self.dataChanged.emit(index, index, [role])
            return True
        else:
            return False

    # Methods for inserting or deleting

    def insertRows(self, position, rows, parent):
        self.beginInsertRows(parent or qtc.QModelIndex(),
                             position, position + rows - 1)

        for i in range(rows):
            default_row = [""] * len(self._headers)
            self._data.insert(position, default_row)
        self.endInsertRows()

    def removeRows(self, position, rows, parent):
        self.beginRemoveRows(parent or qtc.QModelIndex(),
                             position, position + len(rows) - 1)
        for i in rows:
            del self._data[position]
            self.logger.warning(f"Delete {position}")

        self.endRemoveRows()
        self.layoutChanged.emit()

    # method for saving
    def save_data(self):
        with open(self.filename, "w", encoding="utf-8") as fh:
            writer = csv.writer(fh)
            writer.writerow(self._headers)
            writer.writerows(self._data)



from PyQt5 import QtCore
from PyQt5.QtCore import QDate, QDateTime, QRegExp, QSortFilterProxyModel, Qt, QTime
from datetime import datetime

"""
This class is called from imageList tableview/model and  extends 
QSortFilterProxyModel. FilterAcceptRow determines which record
is shown or not based on filters. It parses each row of the
model and determines if the data contained in each field of
the row is matching its filter. 
"""


class SortFilterProxyModel(QSortFilterProxyModel):
    def __init__(self, ui):
        super().__init__()
        self.ui = ui

    def filterAcceptsRow(self, sourceRow, sourceParent):
        indexFile = self.sourceModel().index(sourceRow, 1, sourceParent)
        dataFile = self.sourceModel().data(indexFile)

        indexTarget = self.sourceModel().index(sourceRow, 3, sourceParent)
        dataTarget = self.sourceModel().data(indexTarget)
        regExpTarget = QRegExp(
            self.ui.lineEditTarget.text(), Qt.CaseInsensitive, QRegExp.RegExp
        )

        indexFilter = self.sourceModel().index(sourceRow, 5, sourceParent)
        dataFilter = self.sourceModel().data(indexFilter)
        regExpFilter = QRegExp(
            self.ui.lineEditFilter.text(), Qt.CaseInsensitive, QRegExp.RegExp
        )

        indexFrame = self.sourceModel().index(sourceRow, 4, sourceParent)
        dataFrame = self.sourceModel().data(indexFrame)
        regExpFrame = QRegExp(
            self.ui.lineEditFrame.text(), Qt.CaseInsensitive, QRegExp.RegExp
        )

        indexExposure = self.sourceModel().index(sourceRow, 6, sourceParent)
        if self.sourceModel().data(indexExposure):
            dataExposure = int(self.sourceModel().data(indexExposure))
        operatorExposure = self.ui.comboBoxExposure.itemText(
            self.ui.comboBoxExposure.currentIndex()
        )

        indexFwhm = self.sourceModel().index(sourceRow, 25, sourceParent)
        dataFwhm = self.sourceModel().data(indexFwhm)
        operatorFwhm = self.ui.comboBoxFwhm.itemText(
            self.ui.comboBoxFwhm.currentIndex()
        )

        indexEccentricity = self.sourceModel().index(sourceRow, 26, sourceParent)
        dataEccentricity = self.sourceModel().data(indexEccentricity)
        operatorEccentricity = self.ui.comboBoxEccentricity.itemText(
            self.ui.comboBoxEccentricity.currentIndex()
        )

        indexSnrweight = self.sourceModel().index(sourceRow, 27, sourceParent)
        dataSnrweight = self.sourceModel().data(indexSnrweight)
        operatorSnrweight = self.ui.comboBoxSnrweight.itemText(
            self.ui.comboBoxSnrweight.currentIndex()
        )

        indexNoise = self.sourceModel().index(sourceRow, 28, sourceParent)
        dataNoise = self.sourceModel().data(indexNoise)
        operatorNoise = self.ui.comboBoxNoise.itemText(
            self.ui.comboBoxNoise.currentIndex()
        )

        indexAlt = self.sourceModel().index(sourceRow, 14, sourceParent)
        dataAlt = self.sourceModel().data(indexAlt)
        operatorAlt = self.ui.comboBoxAlt.itemText(
            self.ui.comboBoxAlt.currentIndex())

        indexAz = self.sourceModel().index(sourceRow, 15, sourceParent)
        dataAz = self.sourceModel().data(indexAz)
        operatorAz = self.ui.comboBoxAz.itemText(
            self.ui.comboBoxAz.currentIndex())

        indexDate = self.sourceModel().index(sourceRow, 16, sourceParent)
        dataDate = self.sourceModel().data(indexDate)
        if dataDate:

            dataDate = datetime.strptime(dataDate, "%Y-%m-%dT%H:%M:%S")
        bReturn = True

        if regExpTarget.indexIn(dataTarget) < 0:
            bReturn = False
        if regExpFilter.indexIn(dataFilter) < 0:
            bReturn = False
        if regExpFrame.indexIn(dataFrame) < 0:
            bReturn = False

        if self.ui.lineEditExposure.text() and bReturn == True:
            if (
                operatorExposure == "="
                and int(self.ui.lineEditExposure.text()) == dataExposure
            ):
                bReturn = True
            elif (
                operatorExposure == "<"
                and int(self.ui.lineEditExposure.text()) > dataExposure
            ):
                bReturn = True
            elif (
                operatorExposure == ">"
                and int(self.ui.lineEditExposure.text()) < dataExposure
            ):
                bReturn = True
            else:
                bReturn = False

        if self.ui.doubleSpinBoxEccentricity.value() and bReturn == True:
            if (
                operatorEccentricity == "="
                and self.ui.doubleSpinBoxEccentricity.value() == dataEccentricity
            ):
                bReturn = True
            elif (
                operatorEccentricity == "<"
                and self.ui.doubleSpinBoxEccentricity.value() > dataEccentricity
            ):
                bReturn = True
            elif (
                operatorEccentricity == ">"
                and self.ui.doubleSpinBoxEccentricity.value() < dataEccentricity
            ):
                bReturn = True
            else:
                bReturn = False

        if self.ui.doubleSpinBoxFwhm.value() and bReturn == True:
            if operatorFwhm == "=" and self.ui.doubleSpinBoxFwhm.value() == dataFwhm:
                bReturn = True
            elif operatorFwhm == "<" and self.ui.doubleSpinBoxFwhm.value() > dataFwhm:
                bReturn = True
            elif operatorFwhm == ">" and self.ui.doubleSpinBoxFwhm.value() < dataFwhm:
                bReturn = True
            else:
                bReturn = False

        if self.ui.doubleSpinBoxSnrweight.value() and bReturn == True:
            if (
                operatorSnrweight == "="
                and self.ui.doubleSpinBoxSnrweight.value() == dataSnrweight
            ):
                bReturn = True
            elif (
                operatorSnrweight == "<"
                and self.ui.doubleSpinBoxSnrweight.value() > dataSnrweight
            ):
                bReturn = True
            elif (
                operatorSnrweight == ">"
                and self.ui.doubleSpinBoxSnrweight.value() < dataSnrweight
            ):
                bReturn = True
            else:
                bReturn = False

        if self.ui.doubleSpinBoxNoise.value() and bReturn == True:
            if operatorNoise == "=" and self.ui.doubleSpinBoxNoise.value() == dataNoise:
                bReturn = True
            elif (
                operatorNoise == "<" and self.ui.doubleSpinBoxNoise.value() > dataNoise
            ):
                bReturn = True
            elif (
                operatorNoise == ">" and self.ui.doubleSpinBoxNoise.value() < dataNoise
            ):
                bReturn = True
            else:
                bReturn = False

        if self.ui.lineEditAz.text() and bReturn == True:
            if operatorAz == "=" and int(self.ui.lineEditAz.text()) == dataAz:
                bReturn = True
            elif operatorAz == "<" and int(self.ui.lineEditAz.text()) > dataAz:
                bReturn = True
            elif operatorAz == ">" and int(self.ui.lineEditAz.text()) < dataAz:
                bReturn = True
            else:
                bReturn = False

        if self.ui.lineEditAlt.text() and bReturn == True:
            if operatorAlt == "=" and int(self.ui.lineEditAlt.text()) == dataAlt:
                bReturn = True
            elif operatorAlt == "<" and int(self.ui.lineEditAlt.text()) > dataAlt:
                bReturn = True
            elif operatorAlt == ">" and int(self.ui.lineEditAlt.text()) < dataAlt:
                bReturn = True
            else:
                bReturn = False

        if (
            self.ui.dateEditStartDate.dateTime()
            and self.ui.dateEditEndDate.dateTime()
            and dataDate
            and bReturn == True
        ):
            if (
                self.ui.dateEditStartDate.dateTime() < dataDate
                and self.ui.dateEditEndDate.dateTime() > dataDate
            ):
                bReturn = True
            else:
                bReturn = False

        return bReturn

    def removeSelectedRows(self, indexP, rows, parent):
        for i in sorted(indexP):
            indexSource = self.mapToSource(i)
            self.removeRow(indexSource.row())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'imageDetailWindow.ui'
#
# Created by: PyQt5 UI code generator 5.13.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(755, 1011)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.MplWidget = QtWidgets.QWidget(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.MplWidget.sizePolicy().hasHeightForWidth())
        self.MplWidget.setSizePolicy(sizePolicy)
        self.MplWidget.setMinimumSize(QtCore.QSize(0, 300))
        self.MplWidget.setObjectName("MplWidget")
        self.verticalLayout.addWidget(self.MplWidget)
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        self.groupBox.setObjectName("groupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.labelFile = QtWidgets.QLabel(self.groupBox)
        self.labelFile.setObjectName("labelFile")
        self.gridLayout.addWidget(self.labelFile, 0, 0, 1, 1)
        self.labelHash = QtWidgets.QLabel(self.groupBox)
        self.labelHash.setObjectName("labelHash")
        self.gridLayout.addWidget(self.labelHash, 1, 0, 1, 1)
        self.labelFileTarget = QtWidgets.QLabel(self.groupBox)
        self.labelFileTarget.setObjectName("labelFileTarget")
        self.gridLayout.addWidget(self.labelFileTarget, 2, 0, 1, 1)
        self.lineEditTarget = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditTarget.sizePolicy().hasHeightForWidth())
        self.lineEditTarget.setSizePolicy(sizePolicy)
        self.lineEditTarget.setObjectName("lineEditTarget")
        self.gridLayout.addWidget(self.lineEditTarget, 2, 1, 1, 1)
        self.labelFileDate = QtWidgets.QLabel(self.groupBox)
        self.labelFileDate.setObjectName("labelFileDate")
        self.gridLayout.addWidget(self.labelFileDate, 2, 2, 1, 1)
        self.lineEditDate = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditDate.sizePolicy().hasHeightForWidth())
        self.lineEditDate.setSizePolicy(sizePolicy)
        self.lineEditDate.setObjectName("lineEditDate")
        self.gridLayout.addWidget(self.lineEditDate, 2, 3, 1, 1)
        self.labelFrame = QtWidgets.QLabel(self.groupBox)
        self.labelFrame.setObjectName("labelFrame")
        self.gridLayout.addWidget(self.labelFrame, 3, 0, 1, 1)
        self.lineEditFrame = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditFrame.sizePolicy().hasHeightForWidth())
        self.lineEditFrame.setSizePolicy(sizePolicy)
        self.lineEditFrame.setObjectName("lineEditFrame")
        self.gridLayout.addWidget(self.lineEditFrame, 3, 1, 1, 1)
        self.labelGain = QtWidgets.QLabel(self.groupBox)
        self.labelGain.setObjectName("labelGain")
        self.gridLayout.addWidget(self.labelGain, 3, 2, 1, 1)
        self.lineEditGain = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditGain.sizePolicy().hasHeightForWidth())
        self.lineEditGain.setSizePolicy(sizePolicy)
        self.lineEditGain.setObjectName("lineEditGain")
        self.gridLayout.addWidget(self.lineEditGain, 3, 3, 1, 1)
        self.labelFilter = QtWidgets.QLabel(self.groupBox)
        self.labelFilter.setObjectName("labelFilter")
        self.gridLayout.addWidget(self.labelFilter, 4, 0, 1, 1)
        self.lineEditFilter = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditFilter.sizePolicy().hasHeightForWidth())
        self.lineEditFilter.setSizePolicy(sizePolicy)
        self.lineEditFilter.setObjectName("lineEditFilter")
        self.gridLayout.addWidget(self.lineEditFilter, 4, 1, 1, 1)
        self.labelFileOffset = QtWidgets.QLabel(self.groupBox)
        self.labelFileOffset.setObjectName("labelFileOffset")
        self.gridLayout.addWidget(self.labelFileOffset, 4, 2, 1, 1)
        self.lineEditOffset = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditOffset.sizePolicy().hasHeightForWidth())
        self.lineEditOffset.setSizePolicy(sizePolicy)
        self.lineEditOffset.setObjectName("lineEditOffset")
        self.gridLayout.addWidget(self.lineEditOffset, 4, 3, 1, 1)
        self.labelExposure = QtWidgets.QLabel(self.groupBox)
        self.labelExposure.setObjectName("labelExposure")
        self.gridLayout.addWidget(self.labelExposure, 5, 0, 1, 1)
        self.lineEditExposure = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditExposure.sizePolicy().hasHeightForWidth())
        self.lineEditExposure.setSizePolicy(sizePolicy)
        self.lineEditExposure.setObjectName("lineEditExposure")
        self.gridLayout.addWidget(self.lineEditExposure, 5, 1, 1, 1)
        self.labelSubframeScale = QtWidgets.QLabel(self.groupBox)
        self.labelSubframeScale.setObjectName("labelSubframeScale")
        self.gridLayout.addWidget(self.labelSubframeScale, 5, 2, 1, 1)
        self.lineEditSubframeScale = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSubframeScale.sizePolicy().hasHeightForWidth())
        self.lineEditSubframeScale.setSizePolicy(sizePolicy)
        self.lineEditSubframeScale.setObjectName("lineEditSubframeScale")
        self.gridLayout.addWidget(self.lineEditSubframeScale, 5, 3, 1, 1)
        self.labelTemp = QtWidgets.QLabel(self.groupBox)
        self.labelTemp.setObjectName("labelTemp")
        self.gridLayout.addWidget(self.labelTemp, 6, 0, 1, 1)
        self.lineEditTemp = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditTemp.sizePolicy().hasHeightForWidth())
        self.lineEditTemp.setSizePolicy(sizePolicy)
        self.lineEditTemp.setObjectName("lineEditTemp")
        self.gridLayout.addWidget(self.lineEditTemp, 6, 1, 1, 1)
        self.labelCameraGain = QtWidgets.QLabel(self.groupBox)
        self.labelCameraGain.setObjectName("labelCameraGain")
        self.gridLayout.addWidget(self.labelCameraGain, 6, 2, 1, 1)
        self.lineEditCameraGain = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditCameraGain.sizePolicy().hasHeightForWidth())
        self.lineEditCameraGain.setSizePolicy(sizePolicy)
        self.lineEditCameraGain.setObjectName("lineEditCameraGain")
        self.gridLayout.addWidget(self.lineEditCameraGain, 6, 3, 1, 1)
        self.labelXBinning = QtWidgets.QLabel(self.groupBox)
        self.labelXBinning.setObjectName("labelXBinning")
        self.gridLayout.addWidget(self.labelXBinning, 7, 0, 1, 1)
        self.lineEditXbinning = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditXbinning.sizePolicy().hasHeightForWidth())
        self.lineEditXbinning.setSizePolicy(sizePolicy)
        self.lineEditXbinning.setObjectName("lineEditXbinning")
        self.gridLayout.addWidget(self.lineEditXbinning, 7, 1, 1, 1)
        self.labelCameraResolution = QtWidgets.QLabel(self.groupBox)
        self.labelCameraResolution.setObjectName("labelCameraResolution")
        self.gridLayout.addWidget(self.labelCameraResolution, 7, 2, 1, 1)
        self.lineEditCameraResolution = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditCameraResolution.sizePolicy().hasHeightForWidth())
        self.lineEditCameraResolution.setSizePolicy(sizePolicy)
        self.lineEditCameraResolution.setObjectName("lineEditCameraResolution")
        self.gridLayout.addWidget(self.lineEditCameraResolution, 7, 3, 1, 1)
        self.labelYbinning = QtWidgets.QLabel(self.groupBox)
        self.labelYbinning.setObjectName("labelYbinning")
        self.gridLayout.addWidget(self.labelYbinning, 8, 0, 1, 1)
        self.lineEditYbinning = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditYbinning.sizePolicy().hasHeightForWidth())
        self.lineEditYbinning.setSizePolicy(sizePolicy)
        self.lineEditYbinning.setObjectName("lineEditYbinning")
        self.gridLayout.addWidget(self.lineEditYbinning, 8, 1, 1, 1)
        self.labelScaleUnit = QtWidgets.QLabel(self.groupBox)
        self.labelScaleUnit.setObjectName("labelScaleUnit")
        self.gridLayout.addWidget(self.labelScaleUnit, 8, 2, 1, 1)
        self.lineEditScaleUnit = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditScaleUnit.sizePolicy().hasHeightForWidth())
        self.lineEditScaleUnit.setSizePolicy(sizePolicy)
        self.lineEditScaleUnit.setObjectName("lineEditScaleUnit")
        self.gridLayout.addWidget(self.lineEditScaleUnit, 8, 3, 1, 1)
        self.labelSiteLong = QtWidgets.QLabel(self.groupBox)
        self.labelSiteLong.setObjectName("labelSiteLong")
        self.gridLayout.addWidget(self.labelSiteLong, 9, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEditSiteLong = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSiteLong.sizePolicy().hasHeightForWidth())
        self.lineEditSiteLong.setSizePolicy(sizePolicy)
        self.lineEditSiteLong.setMinimumSize(QtCore.QSize(5, 0))
        self.lineEditSiteLong.setMaximumSize(QtCore.QSize(80, 16777215))
        self.lineEditSiteLong.setObjectName("lineEditSiteLong")
        self.horizontalLayout.addWidget(self.lineEditSiteLong)
        self.lineEditSiteLongF = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditSiteLongF.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSiteLongF.sizePolicy().hasHeightForWidth())
        self.lineEditSiteLongF.setSizePolicy(sizePolicy)
        self.lineEditSiteLongF.setObjectName("lineEditSiteLongF")
        self.horizontalLayout.addWidget(self.lineEditSiteLongF)
        self.gridLayout.addLayout(self.horizontalLayout, 9, 1, 1, 1)
        self.labelDataUnit = QtWidgets.QLabel(self.groupBox)
        self.labelDataUnit.setObjectName("labelDataUnit")
        self.gridLayout.addWidget(self.labelDataUnit, 9, 2, 1, 1)
        self.lineEditDataUnit = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditDataUnit.sizePolicy().hasHeightForWidth())
        self.lineEditDataUnit.setSizePolicy(sizePolicy)
        self.lineEditDataUnit.setObjectName("lineEditDataUnit")
        self.gridLayout.addWidget(self.lineEditDataUnit, 9, 3, 1, 1)
        self.labelSiteLat = QtWidgets.QLabel(self.groupBox)
        self.labelSiteLat.setObjectName("labelSiteLat")
        self.gridLayout.addWidget(self.labelSiteLat, 10, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEditSiteLat = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSiteLat.sizePolicy().hasHeightForWidth())
        self.lineEditSiteLat.setSizePolicy(sizePolicy)
        self.lineEditSiteLat.setMinimumSize(QtCore.QSize(5, 0))
        self.lineEditSiteLat.setMaximumSize(QtCore.QSize(80, 16777215))
        self.lineEditSiteLat.setObjectName("lineEditSiteLat")
        self.horizontalLayout_2.addWidget(self.lineEditSiteLat)
        self.lineEditSiteLatF = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditSiteLatF.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSiteLatF.sizePolicy().hasHeightForWidth())
        self.lineEditSiteLatF.setSizePolicy(sizePolicy)
        self.lineEditSiteLatF.setObjectName("lineEditSiteLatF")
        self.horizontalLayout_2.addWidget(self.lineEditSiteLatF)
        self.gridLayout.addLayout(self.horizontalLayout_2, 10, 1, 1, 1)
        self.labelCsvFile = QtWidgets.QLabel(self.groupBox)
        self.labelCsvFile.setObjectName("labelCsvFile")
        self.gridLayout.addWidget(self.labelCsvFile, 10, 2, 1, 1)
        self.lineEditCsvFile = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditCsvFile.sizePolicy().hasHeightForWidth())
        self.lineEditCsvFile.setSizePolicy(sizePolicy)
        self.lineEditCsvFile.setObjectName("lineEditCsvFile")
        self.gridLayout.addWidget(self.lineEditCsvFile, 10, 3, 1, 1)
        self.labelRa = QtWidgets.QLabel(self.groupBox)
        self.labelRa.setObjectName("labelRa")
        self.gridLayout.addWidget(self.labelRa, 11, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lineEditRa = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditRa.sizePolicy().hasHeightForWidth())
        self.lineEditRa.setSizePolicy(sizePolicy)
        self.lineEditRa.setMinimumSize(QtCore.QSize(10, 0))
        self.lineEditRa.setMaximumSize(QtCore.QSize(80, 16777215))
        self.lineEditRa.setObjectName("lineEditRa")
        self.horizontalLayout_3.addWidget(self.lineEditRa)
        self.lineEditRaF = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditRaF.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditRaF.sizePolicy().hasHeightForWidth())
        self.lineEditRaF.setSizePolicy(sizePolicy)
        self.lineEditRaF.setObjectName("lineEditRaF")
        self.horizontalLayout_3.addWidget(self.lineEditRaF)
        self.gridLayout.addLayout(self.horizontalLayout_3, 11, 1, 1, 1)
        self.labelFwhm = QtWidgets.QLabel(self.groupBox)
        self.labelFwhm.setObjectName("labelFwhm")
        self.gridLayout.addWidget(self.labelFwhm, 11, 2, 1, 1)
        self.lineEditFwhm = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditFwhm.sizePolicy().hasHeightForWidth())
        self.lineEditFwhm.setSizePolicy(sizePolicy)
        self.lineEditFwhm.setObjectName("lineEditFwhm")
        self.gridLayout.addWidget(self.lineEditFwhm, 11, 3, 1, 1)
        self.labelDec = QtWidgets.QLabel(self.groupBox)
        self.labelDec.setObjectName("labelDec")
        self.gridLayout.addWidget(self.labelDec, 12, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.lineEditDec = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditDec.sizePolicy().hasHeightForWidth())
        self.lineEditDec.setSizePolicy(sizePolicy)
        self.lineEditDec.setMaximumSize(QtCore.QSize(80, 16777215))
        self.lineEditDec.setObjectName("lineEditDec")
        self.horizontalLayout_4.addWidget(self.lineEditDec)
        self.lineEditDecF = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditDecF.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditDecF.sizePolicy().hasHeightForWidth())
        self.lineEditDecF.setSizePolicy(sizePolicy)
        self.lineEditDecF.setObjectName("lineEditDecF")
        self.horizontalLayout_4.addWidget(self.lineEditDecF)
        self.gridLayout.addLayout(self.horizontalLayout_4, 12, 1, 1, 1)
        self.labelEccentricity = QtWidgets.QLabel(self.groupBox)
        self.labelEccentricity.setObjectName("labelEccentricity")
        self.gridLayout.addWidget(self.labelEccentricity, 12, 2, 1, 1)
        self.lineEditEccentricity = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditEccentricity.sizePolicy().hasHeightForWidth())
        self.lineEditEccentricity.setSizePolicy(sizePolicy)
        self.lineEditEccentricity.setObjectName("lineEditEccentricity")
        self.gridLayout.addWidget(self.lineEditEccentricity, 12, 3, 1, 1)
        self.labelAlt = QtWidgets.QLabel(self.groupBox)
        self.labelAlt.setObjectName("labelAlt")
        self.gridLayout.addWidget(self.labelAlt, 13, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.lineEditAlt = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditAlt.sizePolicy().hasHeightForWidth())
        self.lineEditAlt.setSizePolicy(sizePolicy)
        self.lineEditAlt.setMaximumSize(QtCore.QSize(80, 16777215))
        self.lineEditAlt.setObjectName("lineEditAlt")
        self.horizontalLayout_5.addWidget(self.lineEditAlt)
        self.lineEditAltF = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditAltF.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditAltF.sizePolicy().hasHeightForWidth())
        self.lineEditAltF.setSizePolicy(sizePolicy)
        self.lineEditAltF.setObjectName("lineEditAltF")
        self.horizontalLayout_5.addWidget(self.lineEditAltF)
        self.gridLayout.addLayout(self.horizontalLayout_5, 13, 1, 1, 1)
        self.labelSnrWeight = QtWidgets.QLabel(self.groupBox)
        self.labelSnrWeight.setObjectName("labelSnrWeight")
        self.gridLayout.addWidget(self.labelSnrWeight, 13, 2, 1, 1)
        self.lineEditSnrWeight = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditSnrWeight.sizePolicy().hasHeightForWidth())
        self.lineEditSnrWeight.setSizePolicy(sizePolicy)
        self.lineEditSnrWeight.setObjectName("lineEditSnrWeight")
        self.gridLayout.addWidget(self.lineEditSnrWeight, 13, 3, 1, 1)
        self.labelAz = QtWidgets.QLabel(self.groupBox)
        self.labelAz.setObjectName("labelAz")
        self.gridLayout.addWidget(self.labelAz, 14, 0, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.lineEditAz = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditAz.sizePolicy().hasHeightForWidth())
        self.lineEditAz.setSizePolicy(sizePolicy)
        self.lineEditAz.setMaximumSize(QtCore.QSize(80, 16777215))
        self.lineEditAz.setObjectName("lineEditAz")
        self.horizontalLayout_6.addWidget(self.lineEditAz)
        self.lineEditAzF = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditAzF.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditAzF.sizePolicy().hasHeightForWidth())
        self.lineEditAzF.setSizePolicy(sizePolicy)
        self.lineEditAzF.setObjectName("lineEditAzF")
        self.horizontalLayout_6.addWidget(self.lineEditAzF)
        self.gridLayout.addLayout(self.horizontalLayout_6, 14, 1, 1, 1)
        self.labelNoise = QtWidgets.QLabel(self.groupBox)
        self.labelNoise.setObjectName("labelNoise")
        self.gridLayout.addWidget(self.labelNoise, 14, 2, 1, 1)
        self.lineEditNoise = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(60)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditNoise.sizePolicy().hasHeightForWidth())
        self.lineEditNoise.setSizePolicy(sizePolicy)
        self.lineEditNoise.setObjectName("lineEditNoise")
        self.gridLayout.addWidget(self.lineEditNoise, 14, 3, 1, 1)
        self.lineEditFile = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditFile.setObjectName("lineEditFile")
        self.gridLayout.addWidget(self.lineEditFile, 0, 1, 1, 3)
        self.lineEditHash = QtWidgets.QLineEdit(self.groupBox)
        self.lineEditHash.setObjectName("lineEditHash")
        self.gridLayout.addWidget(self.lineEditHash, 1, 1, 1, 3)
        self.verticalLayout.addWidget(self.groupBox)
        self.verticalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.groupBox.setTitle(_translate("Dialog", "Image Details"))
        self.labelFile.setText(_translate("Dialog", "File Name"))
        self.labelHash.setText(_translate("Dialog", "File Hash"))
        self.labelFileTarget.setText(_translate("Dialog", "Target"))
        self.labelFileDate.setText(_translate("Dialog", "Obs Date"))
        self.labelFrame.setText(_translate("Dialog", "Frame"))
        self.labelGain.setText(_translate("Dialog", "Gain"))
        self.labelFilter.setText(_translate("Dialog", "Filter"))
        self.labelFileOffset.setText(_translate("Dialog", "Offset"))
        self.labelExposure.setText(_translate("Dialog", "Exposure"))
        self.labelSubframeScale.setText(_translate("Dialog", "Scale"))
        self.labelTemp.setText(_translate("Dialog", "Camera Temp"))
        self.labelCameraGain.setText(_translate("Dialog", "Camera Gain"))
        self.labelXBinning.setText(_translate("Dialog", "X Binning"))
        self.labelCameraResolution.setText(_translate("Dialog", "Camera Res"))
        self.labelYbinning.setText(_translate("Dialog", "Y Binning"))
        self.labelScaleUnit.setText(_translate("Dialog", "Scale Unit"))
        self.labelSiteLong.setText(_translate("Dialog", "Site Longitude"))
        self.labelDataUnit.setText(_translate("Dialog", "Data Unit"))
        self.labelSiteLat.setText(_translate("Dialog", "Site Latitude"))
        self.labelCsvFile.setText(_translate("Dialog", "CSV File"))
        self.labelRa.setText(_translate("Dialog", "RA"))
        self.labelFwhm.setText(_translate("Dialog", "FWHM"))
        self.labelDec.setText(_translate("Dialog", "DEC"))
        self.labelEccentricity.setText(_translate("Dialog", "Eccentricity"))
        self.labelAlt.setText(_translate("Dialog", "ALT"))
        self.labelSnrWeight.setText(_translate("Dialog", "SNR Weight"))
        self.labelAz.setText(_translate("Dialog", "AZ"))
        self.labelNoise.setText(_translate("Dialog", "Noise"))



############################################################
# -*- coding: utf-8 -*-
#
#  Astroimaging catalogue software
#
# Ferrante Enriques
# (c) 2020
#
# License MIT
#
###########################################################
from setuptools import setup, find_packages
import codecs
import os


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            # __version__ = "0.9"
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


long_description = read("README.md")

setup(
    name="astrodom",
    version=get_version("astrodom/__init__.py"),
    packages=find_packages(),
    python_requires=">=3.12",
    install_requires=[
        "astropy",
        "matplotlib",
        "PyQt5",
        "pyqtgraph",
        "numpy",
        "pandas"
    ],
    include_package_data=True,
    url="https://github.com/fenriques/AstroDom",
    license="MIT",
    author="Ferrante Enriques",
    author_email="ferrante.enriques@gmail.com",
    description="Astroimaging Catalogue Software",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: X11 Applications :: Qt",
        "Environment :: Other Environment",
        "Programming Language :: Python :: 3 :: Only",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: POSIX :: Linux",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Astronomy",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
    ],
)
